#include "HLabCompMV.h"
#include "HLabConst.h"

CHLabCompMV::CHLabCompMV(void)  :
	m_pHMM(NULL)
{
}

CHLabCompMV::~CHLabCompMV(void)
{
	if (m_pHMM == NULL)
	{
		delete [] m_pHMM;
	}
}

int CHLabCompMV::ParseCommandLine(int argc, char** argv)
{
	m_compMVConfig.ParseCommandLine(argc, argv);

	return (0);
}

int CHLabCompMV::Init(void)
{
	
    //Reading the phone list file
	m_phoneList.ReadPhoneFile(m_compMVConfig.GetPhoneFile());
//	m_phoneList.ListPhones();

	//Reading the dictionary file
	m_dict.SetPhoneList(&m_phoneList);
	m_dict.ReadDictFile(m_compMVConfig.GetDictFile());
//	m_dict.ListDicWords();
	

	m_pHMM = new CHLabHMM [m_phoneList.GetNumPhones()];

	if (m_pHMM == NULL)
	{
		E_FATAL("Memoray allocation failure\n");
	}

	int i;
	char acBuffer[255];

	for (i = 0; i < m_phoneList.GetNumPhones(); i++)
	{
		sprintf(acBuffer, "%d", i);
		m_pHMM[i].Init(39, m_compMVConfig.GetNumStatesPerHMM(), acBuffer);
	}
	
	m_control.ReadControlFile(m_compMVConfig.GetControlFile());
	//m_control.ListControl();
	
	m_transcription.SetPhoneList(&m_phoneList);
	m_transcription.SetDictList(&m_dict);
//	m_transcription.ReadTranscriptionFile(m_compMVConfig.GetTranscriptionFile());

	m_transcription.Init(m_compMVConfig.GetTranscriptionFile());
	//m_transcription.ListTranscription();

	
	return (0);
}

// Processing the control file line by line
int CHLabCompMV::Run(void)
{
	std::vector<std::string>::iterator itr;
	char acFileName[MAX_LINE];
	FILE* pFile = NULL;

	CHLabFeat feat;

	int iIndex = 0;
	int iRefIndex = 0;
	int n, i, j, k;
	
	
	float afSum[MAX_FEAT_DIM];
	float afSqSum[MAX_FEAT_DIM];
	float afVar[MAX_FEAT_DIM];
	float afMean[MAX_FEAT_DIM];
	int   aiStateCount[MAX_NUM_STATES];
	int  iFeatLen;
	int  iFeatDim;
	int  iStateIndex;
	int  iAccFeatLen;
	int  iAccNumStates;
	float fTrans;

	std::vector<int> vecPhoneSeq;
	std::vector<int>::iterator itrPhoneSeq;

	iFeatLen = m_compMVConfig.GetFeatLen();

	if (iFeatLen <= 0)
	{
		E_FATAL("Invalid feature vector length");
	}

	// Initialization needs to be done

	iAccFeatLen   = 0;
	iAccNumStates = 0;


	
	memset(afSum,   0, sizeof(float) * MAX_FEAT_DIM);
	memset(afSqSum, 0, sizeof(float) * MAX_FEAT_DIM);
	memset(afVar,   0, sizeof(float) * MAX_FEAT_DIM);
	memset(afMean,  0, sizeof(float) * MAX_FEAT_DIM);
	
	
	// Read the control file line by line
	for (itr = m_control.m_vecControl.begin(); itr != m_control.m_vecControl.end(); itr++)
	{
		sprintf(acFileName, "%s/%s.%s", m_compMVConfig.GetFeatDir(),
			    (*itr).c_str(), m_compMVConfig.GetFeatExt());

		printf("%s\n", acFileName);
		
		feat.ReadFeatFile(acFileName, iFeatLen); 

		vecPhoneSeq = m_transcription.ReadTranscriptionFileLine();

		for (itrPhoneSeq = vecPhoneSeq.begin(); itrPhoneSeq < vecPhoneSeq.end(); itrPhoneSeq++)
		{
			iAccNumStates += m_pHMM[*itrPhoneSeq].GetNumStates();

			printf("%d [%d] ", *itrPhoneSeq, m_pHMM[*itrPhoneSeq].GetNumStates());
		}

		
		printf("\n");

		iAccFeatLen += feat.GetLen();
		iFeatDim     = feat.GetFeatDim();


		printf("%d %d %d\n", feat.GetLen(), feat.GetCepDim(), feat.GetFeatDim());

		for (i = 0; i < feat.GetLen(); i++)
		{
			for (j = 0; j < feat.GetFeatDim(); j++)
			{
				afSum[j]   += feat.m_ppfFeat[i][j];
				afSqSum[j] += SQ(feat.m_ppfFeat[i][j]);
			}
		}
	}

	E_INFO("Total number of states : %d\n", iAccNumStates);
	E_INFO("Total number of frames : %d\n", iAccFeatLen);

	if (iAccNumStates <= 0)
	{
		E_FATAL("Invalid number of states");
	}

	if (iAccFeatLen <= 0)
	{
		E_FATAL("Invalid number of feature length");
	}

	for (j = 0; j < feat.GetFeatDim(); j++)
	{
		afMean[j] = afSum[j] / (float)iAccFeatLen;
		afVar[j]  = afSqSum[j] / (float)iAccFeatLen - SQ(afMean[j]);
	}

	fTrans = (float)iAccNumStates / (float)iAccFeatLen;

	
	
	for (n = 0; n < m_phoneList.GetNumPhones(); n++)
	{
		for (i = 0; i < m_pHMM[n].GetNumStates(); i++)
		{
			for (j = 0; j < iFeatDim; j++)
			{
				m_pHMM[n].SetMean(afMean[j], i, j);
				m_pHMM[n].SetVar(afVar[j], i, j);

			}
		}

		/* +1 due to the final non-emitting states */
		for (i = 0; i <= m_pHMM[i].GetNumStates(); i++)
		{
			for (j = 0; j <= m_pHMM[i].GetNumStates(); j++)
			{
				if (i == j && i != m_pHMM[i].GetNumStates())
				{
					m_pHMM[n].SetTrans(1.0 - fTrans, i, j);
				}
				else if (j == i + 1)
				{
					m_pHMM[n].SetTrans(fTrans, i, j);
				}
				else 
				{
					m_pHMM[n].SetTrans(0, i, j);
				}
			}
		}

		m_pHMM[n].WriteHMM(this->m_compMVConfig.GetModelDir());
	}

		

#if 0

		if (iIndex % 5 == 0)
		{
			memset(aafSum,       0, sizeof(float) * 39 * 5);
			memset(aafSqSum,     0, sizeof(float) * 39 * 5);
			memset(aiStateCount, 0, sizeof(int) * 5);
		}

		for (i = 0; i < iFeatLen; i++)
		{
			iStateIndex = i * 5 / iFeatLen;

			aiStateCount[iStateIndex] = aiStateCount[iStateIndex] + 1;

			for (j = 0; j < 39; j++)
			{
				aafSum[iStateIndex][j] = aafSum[iStateIndex][j] + (feat.m_ppfFeat[i][j]);
				aafSqSum[iStateIndex][j] = aafSqSum[iStateIndex][j] + SQ(feat.m_ppfFeat[i][j]);
			}
		}

		if ((iIndex + 1) % 5 == 0)
		{
			for (i = 0; i < 5; i++)
			{
				for (j = 0; j < 39; j++)
				{
					aafMean[i][j] = aafSum[i][j] / aiStateCount[i];

					m_pHMM[iRefIndex].SetMean(aafMean[i][j], i, j);
					
					aafVar[i][j]  = aafSqSum[i][j] / aiStateCount[i] - SQ(aafMean[i][j]);

					aafVar[i][j] = MAX2(aafVar[i][j], 0.0001); /* variance flooring */

					m_pHMM[iRefIndex].SetVar(aafVar[i][j], i, j);

				}
			}

			m_pHMM[iRefIndex].SetDefaultTrans();
			m_pHMM[iRefIndex].WriteHMM(this->m_compMVConfig.GetModelDir());

			//m_pHMM[iRefIndex].ReadHMM(this->m_compMVConfig.GetModelDir());

		}
		iIndex++;

#endif
	
	

	return (0);
}