#include "HLabDTWRecog.h"
#include "HLabConst.h"


CHLabDTWRecog::CHLabDTWRecog(void) :
	m_iNumRef(0),
	m_pRefFeat(NULL),
	m_pDTW(NULL),
	m_piActiveList(NULL)
{
}

CHLabDTWRecog::~CHLabDTWRecog(void)
{
	if (m_pRefFeat != NULL)
	{
		delete [] m_pRefFeat;
	}

	if (m_piActiveList != NULL)
	{
		delete [] m_piActiveList;
	}

	if (m_pDTW != NULL)
	{
		delete [] m_pDTW;
	}
}

int CHLabDTWRecog::ParseCommandLine(int argc, char** argv)
{
	m_compMVConfig.ParseCommandLine(argc, argv);

	return (0);
}

int CHLabDTWRecog::Init(void)
{
	
    //Reading the phone list file
	m_phoneList.ReadPhoneFile(m_compMVConfig.GetPhoneFile());
//	m_phoneList.ListPhones();

	//Reading the dictionary file
	m_dict.SetPhoneList(&m_phoneList);
	m_dict.ReadDictFile(m_compMVConfig.GetDictFile());
//	m_dict.ListDicWords();

	m_control.ReadControlFile(m_compMVConfig.GetControlFile());
	//m_control.ListControl();

	m_iNumRef = m_control.GetLen();

	if (m_iNumRef <= 0)
	{
		fprintf(stderr, "[FATAL] invalid number of references\n");

		exit (0);
	}

	m_pRefFeat = new CHLabFeat [m_iNumRef];

	if (m_pRefFeat == NULL)
	{
		fprintf(stderr, "[FATAL] memory allocation failure\n");

		exit (0);
	}

	printf("Number of refs %d\n", m_iNumRef);
	
	m_transcription.SetPhoneList(&m_phoneList);
	m_transcription.SetDictList(&m_dict);
	m_transcription.ReadTranscriptionFile(m_compMVConfig.GetTranscriptionFile());
	//m_transcription.ListTranscription();

	

	
	return (0);
}

int CHLabDTWRecog::ReadingRef(void)
{
	std::vector<std::string>::iterator itr;
	char acFileName[MAX_LINE];
	FILE* pFile = NULL;

	int iIndex = 0;

	

	for (itr = m_control.m_vecControl.begin(); itr != m_control.m_vecControl.end(); itr++)
	{
		

		sprintf(acFileName, "%s/%s.%s", m_compMVConfig.GetFeatDir(), (*itr).c_str(), m_compMVConfig.GetFeatExt());

	//	printf("%s\n", acFileName);

		m_pRefFeat[iIndex].ReadFeatFile(acFileName, 13); // Temporary hard-wiring

		iIndex++;
	
	}

	return (0);
}

// Processing the control file line by line
int CHLabDTWRecog::Run(void)
{
	// Reading the test feature.

	// Making the time synchronous DTW structure
	int i, j;
	float fBestScore;

	if (m_iNumRef <= 0)
	{
		E_FATAL("Invalid number of reference templates\n");
	}

	m_testFeat.ReadFeatFile(m_compMVConfig.GetInputFile(), 13);

	// Now, let's make the DTW plane.

	m_pDTW         = new CHLabDTW [m_iNumRef];
	m_piActiveList = new int [m_iNumRef];

	for (i = 0; i < m_iNumRef; i++)
	{
		//m_pDTW[i].SetTestFeat();
		//
		m_pDTW[i].SetTestFeat((const float***) &m_testFeat.m_ppfFeat, 
							  m_testFeat.m_iLen, 
				              m_testFeat.m_iFeatDim);

		m_pDTW[i].SetRefFeat((const float***) &(m_pRefFeat[i].m_ppfFeat), 
			                  m_pRefFeat[i].m_iLen,
							  m_pRefFeat[i].m_iFeatDim);


		m_pDTW[i].Init();

		m_piActiveList[i] = 1;

	}

		float fScore;
		float fTempBestScore;

	fBestScore = 1e10;
	

	for (i = 0; i < m_testFeat.m_iLen; i++)
	{
		
		fTempBestScore = 1e10;

		for (j = 0; j < m_iNumRef; j++)
		{
			if (m_piActiveList[j] == 1)
			{

				fScore = m_pDTW[j].Run(fBestScore + m_compMVConfig.GetPruneTH());

				if (fScore < fTempBestScore)
				{
					fTempBestScore = fScore;
				}

				if (fScore > 1e5)
				{
					m_piActiveList[j] = 0;

					//printf("%d is Pruned!\n", j / (m_iNumRef / 10));
				}
			}
		}
		
		fBestScore = fTempBestScore;
	}

	fBestScore = 1e10;

	int iRecogIndex = 0;

	for (i = 0; i < m_iNumRef; i++)
	{
		fScore = m_pDTW[i].GetFinalScore();

		if (fScore < fBestScore)
		{
			fBestScore = fScore;
			iRecogIndex = i;

			
		}
	}

	printf("Recognized index is %d\n", iRecogIndex);

	printf("===========================================\n");
	printf(" [RESULT] %d\n", iRecogIndex / (m_iNumRef / 10));
	printf("===========================================\n");
	

	return (0);
}