#include "HLabAccum.h"
#include "libutil.h"
#include "HLabConst.h"

CHLabAccum::CHLabAccum(void) :
	m_ppfSum(NULL),
	m_ppfSqSum(NULL),
	m_piCount(NULL),
	m_piToCurrent(NULL),
	m_piToNext(NULL),
	m_piToNextNext(NULL),
	m_iDim(0),
	m_iNUmStates(0)
{
}

CHLabAccum::~CHLabAccum(void)
{
	int i;

	if (m_ppfSum != NULL)
	{
		for (i = 0; i < m_iNUmStates; i++)
		{
			delete [] m_ppfSum[i];
		}

		delete [] m_ppfSum;
	}


	if (m_ppfSqSum != NULL)
	{
		for (i = 0; i < m_iNUmStates; i++)
		{
			delete [] m_ppfSqSum[i];
		}

		delete [] m_ppfSqSum;
	}

	if (m_piCount != NULL)
	{
		delete [] m_piCount;
	}

	if (m_piToCurrent != NULL)
	{
		delete [] m_piToCurrent;
	}

	if (m_piToNext != NULL)
	{
		delete [] m_piToNext;
	}

	if (m_piToNextNext != NULL)
	{
		delete [] m_piToNextNext;
	}

}


int CHLabAccum::Init(int iDim, int iNumStates, const char* pszName)
{
	int i;

	m_iDim       = iDim;
	m_iNUmStates = iNumStates;

	m_ppfSum        = new float* [iNumStates];
	m_ppfSqSum      = new float* [iNumStates];
	m_piCount       = new int    [iNumStates];
	m_piToCurrent   = new int    [iNumStates];
	m_piToNext      = new int    [iNumStates];
	m_piToNextNext  = new int    [iNumStates];

	if (m_ppfSum       == NULL ||
		m_ppfSqSum     == NULL ||
		m_piCount      == NULL ||
		m_piToCurrent  == NULL ||
		m_piToNext     == NULL ||
		m_piToNextNext == NULL)
	{
		E_FATAL("Memory allocation failure");
	}

	for (i = 0; i < m_iNUmStates; i++)
	{
		m_ppfSum[i]   = new float [m_iDim];
		m_ppfSqSum[i] = new float [m_iDim];
		
		if (m_ppfSum[i]   == NULL ||
			m_ppfSqSum[i] == NULL)
		{
			E_FATAL("Memory allocation failure");
		}
		
		memset(m_ppfSum[i],   0, sizeof(float) * m_iDim);
		memset(m_ppfSqSum[i], 0, sizeof(float) * m_iDim);
		
	}

	memset(m_piCount,      0, sizeof(int) * iNumStates);
	memset(m_piToCurrent,  0, sizeof(int) * iNumStates);
	memset(m_piToNext,     0, sizeof(int) * iNumStates);
	memset(m_piToNextNext, 0, sizeof(int) * iNumStates);

	return (0);
}

int CHLabAccum::ZeroAccum(void)
{
	int i;

	if (m_ppfSum       == NULL ||
		m_ppfSqSum     == NULL ||
		m_piCount      == NULL ||
		m_piToCurrent  == NULL ||
		m_piToNext     == NULL ||
		m_piToNextNext == NULL)
	{
		E_FATAL("Memory allocation failure");
	}

	for (i = 0; i < m_iNUmStates; i++)
	{
		if (m_ppfSum[i]   == NULL ||
			m_ppfSqSum[i] == NULL)
		{
			E_FATAL("Memory allocation failure");
		}
		
		memset(m_ppfSum[i],   0, sizeof(float) * m_iDim);
		memset(m_ppfSqSum[i], 0, sizeof(float) * m_iDim);
		
	}

	memset(m_piCount,      0, sizeof(int) * m_iNUmStates);
	memset(m_piToCurrent,  0, sizeof(int) * m_iNUmStates);
	memset(m_piToNext,     0, sizeof(int) * m_iNUmStates);
	memset(m_piToNextNext, 0, sizeof(int) * m_iNUmStates);

	return (0);
}

int CHLabAccum::MeanVar(float* fMean, float* fVar, int i, int j)
{

	if (m_ppfSum == NULL || m_ppfSqSum == NULL || m_piCount == NULL)
	{
		E_FATAL("Invalid memoery");
	}

	*fMean = m_ppfSum[i][j]   / (float)m_piCount[i];
	*fVar  = m_ppfSqSum[i][j] / (float)m_piCount[i] - SQ(*fMean);

	return (0);
}
int CHLabAccum::TransProb(float* fToCurrent, 
		                  float* fToNext,
			              float* fToNextNext, int i)
{
	if (m_piCount  == NULL || m_piToCurrent  == NULL || 
		m_piToNext == NULL || m_piToNextNext == NULL)
	{
		E_FATAL("Invalid memoery");
	}

	if ((float)m_piCount[i] > 0)
	{
		*fToCurrent  = ((float)m_piToCurrent[i])  / ((float)m_piCount[i]);
		*fToNext     = ((float)m_piToNext[i])     / ((float)m_piCount[i]);
		*fToNextNext = ((float)m_piToNextNext[i]) / ((float)m_piCount[i]);
	}
	else
	{
		/* If this has not been observed,then just use some default values */
		*fToCurrent  = 0.8;
		*fToNext     = 0.1;
		*fToNextNext = 0.1;
	}
	
	return (0);
}
