/////////////////////////////////////////////////////////////////////////////
//
//	File Name :Config.cpp
//
//													2006
//
//	Author: Chanwoo Kim
//
//  Comments:
//  Oct 22 2008 Added member function 
//
//
//
//
//////////////////////////////////////////////////////////////////////

#include "HLabConfig.h"
//#include "WaveNorm.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

FILE* g_logFile;

const char*  CHLabConfig::DEFAULT_BLOCKSIZE = "200000";

#include <sys/types.h>
#include <sys/stat.h>



arg_t CHLabConfig::m_defn[] = 
{
	{"-phnlstfn",
	ARG_STRING,
	NULL,
	"PHoNe LiST File Name"},

	{"-dictfn",
	ARG_STRING,
	NULL,
	"DICTionary File Name"},

	{"-lsnfn",
	ARG_STRING,
	NULL,
	"Transcription File Name (Format should be in NIST Lexical Standard Normal Orthographic Representation (LSN))"},
	
	{"-alpha",
	ARG_FLOAT64,
	"0.5",
	"The shaping parameter of the gamma distribution for clean speech (Use 0.6 for frame based approach and 0.4 for sample based approach)"},

	{"-srate",
	ARG_FLOAT64,
	"16000.0",
	"The sampling rate of the synapse signal"
	},

	{ "-blocksize",
    ARG_INT32,
    DEFAULT_BLOCKSIZE,			
    "Block size, used to limit the number of samples used at a time when reading very large audio files" },

	{"-c0th",
	ARG_FLOAT64,
	"0.0",
	"Default c0 threshold"},

	{"-silw",
	ARG_FLOAT64,
	"0.4",
	"Default silence weighting",},
	

/*	{"-frame",
	ARG_FLOAT64,
	"little",
	"input endianism"}, */

	{"-i",
	ARG_STRING,
	NULL,
	"Input file name"},


	{"-lamda",
	ARG_FLOAT64,
	"0.99",
	"Default forgetting factor"},

	{"-plamda",
	ARG_FLOAT64,
	"0.9995",
	"Default forgetting factor for peak values (Peak c0/ Peak variance)"},



	{"-in_endian",
	ARG_STRING,
	"little",
	"input endianism"},

	{"-mach_endian",
	ARG_STRING,
	"little",
	"machine endianism"},

	{"-out_endian",
	ARG_STRING,
	"little",
	"output endianism"},

	{"-ifmt",
	ARG_STRING,
	"raw",
	"Input file format (raw)"}, // MS wave does not prase header. It only skips the first 44 bytes.

	{"-lowerf",
	ARG_STRING,
	"130.0",
	"Center frequency of the 0-th channel"},

	{"-upperf",
	ARG_STRING,
	"6800",
	"Center frequency of the highest channel"},

	{"-o",
	ARG_STRING,
	NULL,
	"Output file name"},

	{"-nfft",
	ARG_INT32,
    "1024",
	"fft size"
	},

	{"-featlen",
	ARG_INT32,
	"13",
	"feature vector length"
	},

	{"-vadmvn",
	ARG_INT32,
    "1",
	"VAD MVN"
	},

	{"-wlen",
	ARG_FLOAT64,
    "0.0256",
	"window length"
	},

	{"-fperiod",
	ARG_FLOAT64,
	"0.01",
	"frame period"
	},

	{"-samprate",
	ARG_FLOAT64,
	"16000",
	"sampling rate (Hz)"}, 
	

	{"-ofmt",
	ARG_STRING,
	"raw",
	"Output file format (raw)"}, // MS wave does not prase header. It only skips the first 44 bytes.

	{ "-logfn",
	ARG_STRING,
	NULL,
	"Log file (default stdout/stderr)" },

	
	{"-mtpn",
	ARG_INT32,
	"0",
	"Medium-time power normalization option"
	},

	{"-vad",
	ARG_INT32,
	"1",
	"vad option"
	},

	{"-mvn",
	ARG_INT32,
	"0",
	"Mean variance normalization"
	},

	{"-mpvn",
	ARG_INT32,
	"0",
	"Mean peak variance normalization"
	},

	{"-mvpn",
	ARG_INT32,
	"0",
	"Mean variance c0 peak normalization"
	},

	{"-mpn",
	ARG_INT32,
	"0",
	"Mean variance normalization (do not use.. not working properly)"
	},

	{"-preutt",
	ARG_INT32,
	"0",
	"Use statistics obtained from the previous utterance (do not use PBS!!!)"
	},

	{"-spec",
	ARG_INT32,
	"0",
	"Write the spectrum value instead of writing the feature value",
	},

	{"-spec2cep",
	ARG_INT32,
	"0",
	"Read the PNCC spectrum and generates the feature from it",
	},

	{"-display",
	ARG_INT32,
	"0",
	"standard deviation is normalized."
	},



	{ NULL, 0,  NULL, NULL }
};

CHLabConfig::CHLabConfig() :
	m_pszPhoneFile(NULL),
	m_pszDictFile(NULL),
	m_pszLSNFile(NULL),
	m_pszControlFile(NULL),
	m_pszFeatDir(NULL),
	m_pszFeatExt(NULL),
	m_pszBufferDir(NULL),
	m_pszModelDir(NULL),
	m_iFeatLen(DEFAULT_FEAT_LEN)
{
	m_pszLogFileName    = NULL;
	m_pszInWaveFileName = NULL;
	m_pszHistFile       = NULL;
	
	// Addition: Aug. 26, 2007


	
	m_pszAnalFilterBank = NULL;
	m_pszSynFilterBank  = NULL;

	m_dWindowLen   = 0.0;
	m_dFramePeriod = 0.0;

	m_bSpec     = 0;
	m_bSpec2cep = 0;

	// Temporarily hard-wired.....
	m_iNumFilts = 40;

	m_iFFTSize = 0;

	g_logFile = NULL;
}

CHLabConfig::~CHLabConfig()
{
	if (g_logFile != NULL)
	{
		fclose (g_logFile);
	}
}

///////////////////////////////////////////////////////////////////////
//
// Function name ParseCommandLine
//
// Parameter
//  * argc (i)
//  * argv (i)
//
// Return value
//  *
//

int32 CHLabConfig::ParseCommandLine(int argc, char** argv)
{
	print_appl_info(argv[0]);

	cmd_ln_appl_enter(argc, argv, "default.arg", m_defn);

	
	m_pszOutFeatFile     = cmd_ln_str("-o");
	m_pszInWaveFile      = cmd_ln_str("-i");
	m_pszInFormat        = cmd_ln_str("-ifmt");

	m_pszInEndian         = cmd_ln_str("-in_endian");
	m_pszOutEndian        = cmd_ln_str("-out_endian");
	m_pszMachEndian       = cmd_ln_str("-mach_endian");

	m_dWindowLen        = cmd_ln_float64("-wlen");
	m_dFramePeriod      = cmd_ln_float64("-fperiod");
	m_dSampRate         = cmd_ln_float64("-samprate");

	m_dc0th            = cmd_ln_float64("-c0th"); // c0 threshold
	m_dSilWeighting    = cmd_ln_float64("-silw");

	m_dLamda           = cmd_ln_float64("-lamda");

	//m_dPLamda          = cmd_ln_float64("-plamda");

	m_iBlockSize		= cmd_ln_int32("-blocksize");

	m_iFFTSize			= cmd_ln_int32("-nfft");

	m_iVAD              = cmd_ln_int32("-vad");
	m_iMTPN             = cmd_ln_int32("-mtpn");

	m_iMVN              = cmd_ln_int32("-mvn");
	m_iMPN              = cmd_ln_int32("-mpn");
	m_iMVPN             = cmd_ln_int32("-mvpn");
	m_iMPVN             = cmd_ln_int32("-mpvn");
	m_iMVNVAD           = cmd_ln_int32("-vadmvn");

	m_iDisplay          = cmd_ln_int32("-display");

	m_iPreUtt           = cmd_ln_int32("-preutt");

	m_bSpec             = cmd_ln_int32("-spec");
	m_bSpec2cep         = cmd_ln_int32("-spec2cep");


	printf("%d\n", m_iBlockSize);

	print_appl_info(argv[0]);

	return (0);
}



int32 CHLabConfig::Init(void)
{
#ifdef WIN32
	struct _stat fileStat;
#else
	struct stat  fileStat;
#endif
	int32        iResult;
	int32        iNumRetry;

	if (m_pszInWaveFileName == NULL)
	{
		fprintf(stderr, "The input file name should be specified.!\n");

		exit (0);
	}

#ifdef WIN32
	iResult = _stat(m_pszInWaveFileName, &fileStat);
#else
	iResult = stat(m_pszInWaveFileName, &fileStat);
#endif


	if (strcmp(this->m_pszInFormat, "nist") == 0)
		this->m_iTotalSpeechLen = (fileStat.st_size - 1024) / sizeof(short);
	else
		this->m_iTotalSpeechLen = (fileStat.st_size) / sizeof(short);

	printf ("%d", m_iBlockSize);


	//m_iNumBlocks = m_iSpeechLen / m_iBlockSize;



	if (m_pszLogFileName != NULL)
	{
		g_logFile = fopen(m_pszLogFileName, "wt");

		if (g_logFile == NULL)
		{
			E_FATAL("[ERROR] Log file open error!\n");
		}
	}
	else
	{
		E_INFO("Logfile will not be used.");
	}

	// Checking about the filter bank case

		
	return (0);
}

