// Config.h: interface for the CHLabConfig class.
//
//////////////////////////////////////////////////////////////////////

/**
 * @file
 * @author  Chanwoo Kim (chanwook@cs.cmu.edu)
 * @version 1.0
 *
 * @section LICENSE
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details at
 * http://www.gnu.org/copyleft/gpl.html
 *
 * @section DESCRIPTION
 *
 * 
 */


#pragma once

//#include "s3types.h"
//#include "fe.h"
#include "libutil.h"
#include "HLabConst.h"

#ifdef WIN32
	#if defined _USRDLL 
		#define DllExport _declspec(dllexport)
	#else
		#define DllExport _declspec(dllimport)
	#endif 
#else
	#define DllExport
#endif


class DllExport CHLabConfig  
{
private:
	static arg_t m_defn[];

protected:
	//**************************************//
	const char* m_pszPhoneFile;
	const char* m_pszDictFile;
	const char* m_pszLSNFile;
	const char* m_pszControlFile;
	const char* m_pszFeatDir;
	const char* m_pszFeatExt;
	const char* m_pszBufferDir;
	const char* m_pszModelDir;
	int         m_iFeatLen;
	int         m_iNumStatesPerHMM;

public:

	 /**
    * Constructor that sets the time to a given value.
    * 
    * @param timemillis Number of milliseconds 
    *        passed since Jan 1, 1970.
    */


	CHLabConfig();
	virtual ~CHLabConfig();

public:

	inline const char* GetPhoneFile()
	{
		return m_pszPhoneFile;
	}
	
	inline const char* GetDictFile()
	{
		return m_pszDictFile;
	}

	inline const char* GetControlFile()
	{
		return m_pszControlFile;
	}

	inline const char* GetTranscriptionFile()
	{
		return m_pszLSNFile;
	}

	inline const char* GetFeatDir()
	{
		return m_pszFeatDir;
	}

	inline const char* GetFeatExt()
	{
		return m_pszFeatExt;
	}

	inline const char* GetModelDir()
	{
	    return m_pszModelDir;
	}

	inline int GetFeatLen()
	{
		return m_iFeatLen;
	}


	
	inline int GetNumStatesPerHMM()
	{
		return m_iNumStatesPerHMM;
	}



	/*                       */

	// -param (Parametric histrogram normalization)
	int     m_iParam;

	// -alpha
	float64 m_dAlpha;
	// -srate
	float64 m_dSampRate;
	// -blocks
	int32   m_iBlockSize;

	// -vad option
	int32   m_iVAD;

	// -MTPN option
	int32   m_iMTPN;

	// -mvn option
	int32   m_iMVN;

	// -mpn option
	int32   m_iMPN;

	int32   m_iMVPN;

	int32   m_iMPVN;

	// -display option
	int32   m_iDisplay;

	float64 m_dLamda;
	float64 m_dPLamda;

	int32   m_iPreUtt;

	int32   m_iMVNVAD;

	//////////////////////////////////////////////////////////////////////
	//
	// Added for missing feature experiment...
	//
	int32   m_bSpec;
	int32   m_bSpec2cep;

	

	// -ength option
	float64 m_dc0th;

	float64 m_dSilWeighting;

	// input file name
	const char*   m_pszInWaveFileName;
	// -input_endian
	const char*   m_pszInEndian;
	// -ifmt
	const char*   m_pszInFormat;

	



	

	int32 m_iNumFilts;


	const char*   m_pszHistFile;

	const char*   m_pszInWaveFile;

	// -o
	const char*   m_pszOutFeatFile;
	// -output_endian
	const char*   m_pszOutEndian;
	// -ofmt

	const char*   m_pszMachEndian;


	const char*   m_pszOutFormat;

	// -logfn
	const char*   m_pszLogFileName;

	static const char* DEFAULT_BLOCKSIZE;

		
	const char* m_pszAnalFilterBank;
	const char* m_pszSynFilterBank;
	
	int32 m_iTotalSpeechLen;
	int32 m_iNumBlocks;
	int32 m_iSpeechLen;

	int32 m_iFFTSize;
	


	float64 m_dWindowLen;
	float64 m_dFramePeriod;

	

	int32 Init(void);
	int32 ParseCommandLine(int argc, char** argv);
};


