#include "HLabDTW.h"



#define SQ(X) ((X) * (X))

#define MAX2(X, Y) (((X) > (Y))? (X) : (Y))
#define MAX3(X, Y, Z) (MAX2(MAX2(X, Y), Z))

#define MIN2(X, Y) (((X) < (Y))? (X) : (Y))
#define MIN3(X, Y, Z) (MIN2(MIN2(X, Y), Z))

CHLabDTW::CHLabDTW(void) :
	m_pppfTestFeat(NULL),
	m_pppfRefFeat(NULL),
	m_iTestFeatLen(0),
	m_iRefFeatLen(0),
	m_ppfScoreTable(NULL),
	m_iT(0),
	m_fBestScore(0.0),
	m_pfLocalDist(NULL),
	INF(1e10)
{
}

CHLabDTW::~CHLabDTW(void)
{
	int i;

	if (m_ppfScoreTable != NULL)
	{
		for (i = 0; i < m_iTestFeatLen; i++)
		{
			delete [] m_ppfScoreTable[i];			
		}
		delete [] m_ppfScoreTable;
	}

	if (m_pfLocalDist != NULL)
	{
		delete [] m_pfLocalDist;
	}
}

int CHLabDTW::Init()
{
	int i, j;

	if (m_iTestFeatDim != m_iRefFeatDim)
	{
		E_FATAL("Test and reference feature lengths do not match.\n");
	}

	//printf("[%d, %d]\n", m_iRefFeatLen, m_iTestFeatLen);

	m_ppfScoreTable = new float* [m_iTestFeatLen];

	if (m_ppfScoreTable == NULL)
	{
		E_FATAL("memory allocation error\n");
	}

	for (i = 0; i < m_iTestFeatLen; i++)
	{
		m_ppfScoreTable[i] = new float [m_iRefFeatLen];

		if (m_ppfScoreTable[i] == NULL)
		{
			E_FATAL("memory allocation error\n");
		}

		for (j = 0; j < m_iRefFeatLen; j++)
		{
			m_ppfScoreTable[i][j] = INF;
		}
	}

	m_pfLocalDist = new float [m_iRefFeatLen];

	if (m_pfLocalDist == NULL)
	{
		E_FATAL("memory allocation error\n");
	}
	
	m_iT         = 0;
	m_fBestScore = 0.0;

	

	return (0);
}

float CHLabDTW::Run(float fTH)
{
	/* Computation of the local distance */
	int i;
	float fDist1;
	float fDist2;
	float fDist3;
	int   bFlag;
	float fLocal;
	float fBest;

	fBest = INF;
	

	memset(m_pfLocalDist, 0, sizeof(float) * m_iRefFeatLen);

	if (m_iT == 0)
	{
		m_ppfScoreTable[m_iT][0] = EuclideanDistance(m_iT, 0);

		fBest = m_ppfScoreTable[m_iT][0];
	}
	else
	{
		for (i = 0; i < m_iRefFeatLen; i++)
		{
			fDist1 = INF;
			fDist2 = INF;
			fDist3 = INF;

			bFlag = 0;


			if (i - 2 >= 0)
			{
				if (m_ppfScoreTable[m_iT - 1][i - 2] < fTH)
				{
					bFlag = 1;
				}
			}
			if (i - 1 >= 0)
			{
				if (m_ppfScoreTable[m_iT - 1][i - 1] < fTH)
				{
					bFlag = 1;
				}
			}
			{
				if (m_ppfScoreTable[m_iT - 1][i] < fTH)
				{
					bFlag = 1;
				}
			}

			if (bFlag == 1)
			{
				fLocal = EuclideanDistance(m_iT, i);

				if (i - 2 >= 0)
				{
					fDist1 = m_ppfScoreTable[m_iT - 1][i - 2] + fLocal;
					
				}
				if (i - 1 >= 0)
				{
					fDist2 = m_ppfScoreTable[m_iT - 1][i - 1] + fLocal;
				}
				{
					fDist3 = m_ppfScoreTable[m_iT - 1][i]     + fLocal;
				}

				m_ppfScoreTable[m_iT][i] = MIN3(fDist1, fDist2, fDist3);

				if (m_ppfScoreTable[m_iT][i] < fBest)
				{
					fBest = m_ppfScoreTable[m_iT][i];
				}
			}
		}


	}

	m_iT++;

	return (fBest);
}

inline float CHLabDTW::EuclideanDistance(int iTestFrameIndex, int iRefFrameIndex)
{
	float fDist;
	int i;

	fDist = 0.0;

	
	for (i = 0; i < m_iTestFeatDim; i++)
		fDist += SQ((*m_pppfTestFeat)[iTestFrameIndex][i] - (*m_pppfRefFeat)[iRefFrameIndex][i]);


	return (fDist);
}

float CHLabDTW::GetFinalScore(void)
{
	if (m_ppfScoreTable == NULL)
	{
		E_FATAL("Score table is not set");

	}

	return m_ppfScoreTable[m_iTestFeatLen - 1][m_iRefFeatLen - 1];
}
