#include <sys/types.h>
#include <sys/stat.h>


#include "HLabFeat.h"


#include "libutil.h"
#include "byteorder.h"

#define MAX2(X, Y) (((X) > (Y))? (X) : (Y))
#define MAX3(X, Y, Z) (MAX2(MAX2(X, Y), Z))

#define MIN2(X, Y) (((X) < (Y))? (X) : (Y))
#define MIN3(X, Y, Z) (MIN2(MIN2(X, Y), Z))

CHLabFeat::CHLabFeat() : 
	m_ppfFeat(NULL),
	m_bByteSwap(0),
	m_iLen(0),
	m_iCepDim(13),
	m_iFeatDim(39)
{
}

CHLabFeat::~CHLabFeat()
{
	int i;

	if (m_ppfFeat != NULL)
	{
		// memory de-allocation
		for (i = 0; i < m_iLen; i++)
		{
			delete [] m_ppfFeat[i];
		}

		delete [] m_ppfFeat;
	}
}

int CHLabFeat::ReadFeatFile(const char* pszFile, int iDim)
{
	FILE* m_pFeatFile = NULL;
	int  iNumData;
	int  iHeaderSize;
	int i, j, iResult;
	int iPos1, iPos2, iWin;

	if (m_ppfFeat != NULL)
	{
		// memory de-allocation
		for (i = 0; i < m_iLen; i++)
		{
			delete [] m_ppfFeat[i];
		}

		delete [] m_ppfFeat;
	}
	
	m_bByteSwap = 0;
	m_iLen      = 0;
	
	m_iCepDim  = iDim; 
	/* fix me -- hard coding */
	m_iFeatDim = m_iCepDim * 3;


	


	m_pFeatFile = fopen(pszFile, "rb");

	
	if (m_pFeatFile == NULL)
	{
		E_FATAL("File open error");
	}

	// Let's check the file size.. How?

	iNumData = (GetFileSize(pszFile) - 4) / 4;
	

	fread(&iHeaderSize, sizeof(int32), 1, m_pFeatFile);

	if (iHeaderSize == iNumData)
	{
		m_bByteSwap = 0;
	}
	else if (SWAP_INT32(&iHeaderSize) == iNumData)
	{
		m_bByteSwap = 1;
	}
	else
	{
		E_FATAL("Header error. File size and header do not match.");
	}

	m_iLen = iNumData / m_iCepDim;

	// Memory allocation for feature

	m_ppfFeat = new float* [m_iLen];

	if (m_ppfFeat == NULL)
	{
		E_FATAL("Memory allocation failure\n");
	}

	

	for (i = 0; i < m_iLen; i++)
	{
		m_ppfFeat[i] = new float [m_iCepDim * 3];

		if (m_ppfFeat[i] == NULL)
		{
			E_FATAL("Memory allocation failure\n");
		}
	}

	for (i = 0; i < m_iLen; i++)
	{
		iResult = fread(m_ppfFeat[i], sizeof(float32), m_iCepDim, m_pFeatFile);

		if (m_bByteSwap == 1)
		{
			for (j = 0; j < m_iCepDim; j++)
			{
				SWAP_FLOAT32(&m_ppfFeat[i][j]);
			}
		}
	}

	

	
		

	fclose(m_pFeatFile);

	/* Silence is causing the problem */

	/* Let's do Cepstral Mean Normalization */

	float fSum;

	for (i = 0; i < m_iCepDim; i++)
	{
		fSum = 0.0;

		for (j = 0; j < m_iLen; j++)
		{
			fSum += m_ppfFeat[j][i];
		}

		fSum /= (float) m_iLen;

		for (j = 0; j < m_iLen; j++)
		{
			m_ppfFeat[j][i] -= fSum;
		}
	}


	/* Delta feature calculation */
	for (i = 0; i < m_iLen; i++)
	{
		iPos1 = MAX2(i - 2, 0);
		iPos2 = MIN2(i + 2, m_iLen - 1);
		iWin  = iPos2 - iPos1;
		

		for (j = 0; j < m_iCepDim; j++)
		{
			m_ppfFeat[i][j + m_iCepDim] = (m_ppfFeat[iPos2][j] - m_ppfFeat[iPos1][j]) / (float)iWin * 4.0;
		}
		
	}

	/* Delta Delta feature calculation */
	for (i = 0; i < m_iLen; i++)
	{
		iPos1 = MAX2(i - 2, 0);
		iPos2 = MIN2(i + 2, m_iLen - 1);
		iWin  = iPos2 - iPos1;

		for (j = 0; j < m_iCepDim; j++)
		{
			m_ppfFeat[i][j + 2 * m_iCepDim] = (m_ppfFeat[iPos2][j + m_iCepDim] - m_ppfFeat[iPos1][j + m_iCepDim]) / (float)iWin * 4.0;
		}
	}

	return (0);
}


int32 CHLabFeat::GetFileSize(const char* pszFeatFile)
{
#ifdef WIN32
	struct _stat fileStat;
#else
	struct stat fileStat;
#endif
    int iLen;

    

#ifdef WIN32
	_stat(pszFeatFile, &fileStat);
#else
	stat(pszFeatFile, &fileStat);
#endif



    
    
	return (fileStat.st_size);
}
