#include "HLabFeatOld.h"

#include <stdio.h>

#include "fe.h"
#include "libutil.h"
#include "byteorder.h"



#include <sys/types.h>
#include <sys/stat.h>

#if !defined WIN32
	#include <unistd.h>
#endif


///////////////////////////////////////////////////////////////////////
//
// Feature class
//
// Designed and programmed by Chanwoo kim
//


CFeature::CFeature()
{
	m_pszFeatFile = NULL;
	m_pFeatFile   = NULL;
	m_iDim        = 13; // Default value
	m_iNumVec     = 0;
	m_iByteSwap   = 0;
}

CFeature::~CFeature()
{
	if (m_pFeatFile != NULL)
	{
		WriteHeader();

		fclose(m_pFeatFile);
	}
}

///////////////////////////////////////////////////////////////////////
//
// Init
//
int32 CFeature::Init(const char* pszFeatFile, int iDim, FEAT_MODE featMode)
{
	m_pszFeatFile = pszFeatFile;
	m_iDim = iDim;
	int iNumData;
	int iHeaderSize;

	// Let's write the file..

	m_featMode = featMode;

	if (m_featMode == FEAT_WRITE)
	{
		m_pFeatFile = fopen(m_pszFeatFile, "wb");

		if (m_pFeatFile == NULL)
		{
			E_FATAL("File open error");
		}

		//
		// We don't know the size.. so let's just skip the first 4 bytes..
		// 
		// Skipped part will be written later
		//
		fseek(m_pFeatFile, 4, SEEK_SET);
	}
	else if (m_featMode == FEAT_READ)
	{
		m_pFeatFile = fopen(m_pszFeatFile, "rb");

		
		if (m_pFeatFile == NULL)
		{
			E_FATAL("File open error");
		}

		// Let's check the file size.. How?

		iNumData = (GetFileSize(m_pszFeatFile) - 4) / 4;
		

		fread(&iHeaderSize, sizeof(int32), 1, m_pFeatFile);

		if (iHeaderSize == iNumData)
		{
			m_iByteSwap = 0;
		}
		else if (SWAP_INT32(&iHeaderSize) == iNumData)
		{
			m_iByteSwap = 1;
		}
		else
		{
			E_FATAL("Header error. File size and header do not match.");
		}

	}
	else
	{
		E_FATAL("Undefined mode");
	}

	return (0);
}

/////////////////////////////////////////////////////////////////////////////
//
//
int32 CFeature::GetFileSize(const char* pszFeatFile)
{
#ifdef WIN32
	struct _stat fileStat;
#else
	struct stat fileStat;
#endif
    int iLen;

    

#ifdef WIN32
	_stat(pszFeatFile, &fileStat);
#else
	stat(pszFeatFile, &fileStat);
#endif



    
    
	return (fileStat.st_size);
}

int32 CFeature::WriteHeader()
{
	int iNumData;

	iNumData = m_iNumVec * m_iDim;

	printf("Total number of data is %d\n", iNumData);

	fseek(m_pFeatFile, 0, SEEK_SET);

	
	fwrite(&iNumData, sizeof(int), 1, m_pFeatFile);


	return (0);

}

int32 CFeature::CloseFile(void)
{
	if (m_pFeatFile != NULL)
	{
		fclose(m_pFeatFile);
	}

	m_pFeatFile = NULL;

	return (0);
}


int32 CFeature::WriteVec(float32* pfVec)
{
	m_iNumVec++;

	if (m_pFeatFile == NULL)
	{
		E_FATAL("Invalid file pointer");
	}

	fwrite(pfVec, sizeof(float32), m_iDim, m_pFeatFile);

	return (0);
}

int32 CFeature::ReadVec(float32* pf_x, int32 i_x_Len)
{
	int iResult;
	int i;

	if (i_x_Len != m_iDim)
	{
		E_FATAL("Feature size does not match.\n");
	}

	if (m_pFeatFile == NULL)
	{
		E_FATAL("Invalid file pointer");
	}


	iResult = fread(pf_x, sizeof(float32), i_x_Len, m_pFeatFile);

	if (m_iByteSwap == 1)
	{
		for (i = 0; i < i_x_Len; i++)
		{
			SWAP_FLOAT32(&pf_x[i]);
		}
	}

	return (iResult);
}

#if 0
int32 CFeature::WriteMatrix(float32** ppfVec, int m_)
{
}
#endif
