#include "HLabHMM.h"

#include <math.h>

#include "libutil.h"
#include "HLabConst.h"

#define MAX2(X, Y) (((X) > (Y))? (X) : (Y))

/**
 * @file
 * @author  Chanwoo Kim
 * @version 1.0
 *
 * @section LICENSE
 *
 * 
 *
 * @section DESCRIPTION
 *
 * @version
 *   March, 29, 2010 include non-emitting state
 * 
 */


CHLabHMM::CHLabHMM(void) :
	m_ppfMean(NULL),
	m_ppfVar(NULL),
	m_ppfTrans(NULL),
	m_pszName(NULL)
{
}

CHLabHMM::~CHLabHMM(void)
{
	int i;

#if 1
	if (m_ppfMean != NULL)
	{
		for (i = 0; i < m_iNumStates; i++)
		{
			delete [] m_ppfMean[i];
		}

		delete [] m_ppfMean;
	}

	if (m_ppfVar != NULL)
	{
		for (i = 0; i < m_iNumStates; i++)
		{
			delete [] m_ppfVar[i];
		}

		delete [] m_ppfVar;
	}

	if (m_ppfTrans != NULL)
	{
		for (i = 0; i < m_iNumStates + 1; i++)
		{
			delete [] m_ppfTrans[i];
		}

		delete [] m_ppfTrans;
	}

	if (m_pszName != NULL)
	{
		delete [] m_pszName;
	}

#endif
}

int CHLabHMM::SetDefaultTrans(void)
{
	int i, j;

	for (i = 0; i < m_iNumStates + 1; i++)
	{
		for (j = 0; j < m_iNumStates + 1; j++)
		{
			if (i == j) 
			{
				m_ppfTrans[i][j] = (float)(9.0 / 10.0);// + 0.001 * rand();
			}
			else if (j == i + 1)
			{
				m_ppfTrans[i][j] = (float)(1.0/ 10.0); //+ 0.001 * rand();
			}
			else if (j == i + 2)
			{
				m_ppfTrans[i][j] = (float)(0.0 / 10.0); // + 0.001 * rand();
			}
			else
			{
				m_ppfTrans[i][j] = 0;
			}
		}
	}

	/* For the case of the final non-emitting states */
	m_ppfTrans[m_iNumStates][m_iNumStates] = 0.0;

	return (0);
}

int CHLabHMM::Init(int iDim, int iNumStates, const char* pszName)
{
	int i, j;

	m_iDim       = iDim;
	m_iNumStates = iNumStates;
	

	m_ppfMean = new float* [iNumStates];

	if (m_ppfMean == NULL)
	{
		E_FATAL("Memory allocation error\n");
	}

	for (i = 0; i < iNumStates; i++)
	{
		m_ppfMean[i] = new float [m_iDim];

		if (m_ppfMean[i] == NULL)
		{
			E_FATAL("Memory allocation error\n");
		}
	}

	m_ppfVar = new float* [iNumStates];


	if (m_ppfVar == NULL)
	{
		E_FATAL("Memory allocation error\n");
	}

	for (i = 0; i < iNumStates; i++)
	{
		m_ppfVar[i] = new float [m_iDim];

		if (m_ppfVar[i] == NULL)
		{
			E_FATAL("Memory allocation error\n");
		}
	}

	m_ppfTrans = new float* [iNumStates + 1];

	for (i = 0; i < iNumStates + 1; i++)
	{
		m_ppfTrans[i] = new float [iNumStates + 1];
	}

	m_pszName = new char[strlen(pszName) + 1];

	strcpy(m_pszName, pszName);

	
	return (0);
}

int CHLabHMM::WriteHMM(const char* szDir)
{
	char acFileName[MAX_LINE];

	sprintf(acFileName, "%s/%s.txt", szDir, m_pszName);

	printf("%s\n", acFileName);

	int i, j;

	FILE* pFile = NULL;
	
	pFile = fopen(acFileName, "wt");

	if (pFile == NULL)
	{
		E_FATAL("File cannot be opened.\n");
	}

	fprintf(pFile, "<HMM>\n");

	for (i = 0; i < m_iNumStates; i++)
	{
		fprintf(pFile, "<STATE_%d>\n", i);
		
		fprintf(pFile, "<MEAN>\n");
		for (j = 0; j < m_iDim ; j++)
		{
			fprintf(pFile, "%10.8f ", m_ppfMean[i][j]);
		}

		fprintf(pFile, "\n");
		fprintf(pFile, "<VAR>\n");
		for (j = 0; j < m_iDim ; j++)
		{
			fprintf(pFile, "%10.8f ", m_ppfVar[i][j]);
		}
		fprintf(pFile, "\n");


		
	}

	fprintf(pFile, "<TRANS>\n");
	for (i = 0; i < m_iNumStates + 1; i++)
	{
		for (j = 0; j < m_iNumStates + 1; j++)
		{

			fprintf(pFile, "%10.8f ", log(MAX2(m_ppfTrans[i][j], 1e-100)));

		}
		fprintf(pFile, "\n");
	}

	


	fprintf(pFile, "<\\HMM>\n");
	

	fclose (pFile);

	return (0);
}

int CHLabHMM::ReadHMM(const char* szDir)
{

	char acFileName[MAX_LINE];
	char acBuffer[MAX_LINE];

	sprintf(acFileName, "%s/%s.txt", szDir, m_pszName);

	//printf("%s\n", acFileName);

	int i, j;

	FILE* pFile = NULL;
	
	pFile = fopen(acFileName, "rt");

	if (pFile == NULL)
	{
		E_FATAL("File cannot be opened.\n");
	}

	fgets(acBuffer, MAX_LINE, pFile);

	for (i = 0; i < m_iNumStates; i++)
	{
		fgets(acBuffer, MAX_LINE, pFile);
		
		fgets(acBuffer, MAX_LINE, pFile);
		for (j = 0; j < m_iDim ; j++)
		{
			fscanf(pFile, "%f ", &m_ppfMean[i][j]);
		}

		fgets(acBuffer, MAX_LINE, pFile);

		for (j = 0; j < m_iDim ; j++)
		{
			fscanf(pFile, "%f ", &m_ppfVar[i][j]);
		}

		
	}

	fgets(acBuffer, MAX_LINE, pFile);
	for (i = 0; i < m_iNumStates + 1; i++)
	{
		for (j = 0; j < m_iNumStates + 1; j++)
		{
			fscanf(pFile, "%f ", &m_ppfTrans[i][j]);
		}
		//fgets(acBuffer, MAX_LINE, pFile);
	}

	fgets(acBuffer, MAX_LINE, pFile);
	

	fclose (pFile);

	return (0);
}



