#include <iostream>

#include "HLabLM.h"
#include "HLabConst.h"

#include "libutil.h"


CHLabLM::CHLabLM(void)
{
}

CHLabLM::~CHLabLM(void)
{
}

LM_PROB CHLabLM::ComputeTrigramProb(std::string strTrigram)
{
	/* Let's check whether we can find the trigram */
	LM_PROB LMProb;
	float fLike;
	float fBackoff, fBigram;
	
	char strBuffer[MAX_LINE_LEN];
	const char* pszToken1 = NULL;
	const char* pszToken2 = NULL;
	const char* pszToken3 = NULL;

	char acBackoff[MAX_LINE_LEN];
	char acBigram[MAX_LINE_LEN];

	
	char seps[]   = "_";

	LMProb = m_hashTrigram[strTrigram];
	fLike  = LMProb.m_fLogProb;

	/* Serach in the bigram */
	if (fLike == 0)
	{
		std::string strTemp = strTrigram;

		pszToken1 = strtok((char*)strTemp.c_str(), seps);
		pszToken2 = strtok(NULL, seps);
		pszToken3 = strtok(NULL, seps);

		//printf("%s   word1 %s word2 %s word3 %s\n", strTrigram.c_str(), 
		//	pszToken1, pszToken2, pszToken3);

		sprintf(acBackoff, "%s_%s", pszToken2, pszToken1);

		sprintf(acBigram, "%s_%s", pszToken3, pszToken2);

		//printf("%s %s\n\n", acBackoff, acBigram);

		LMProb = ComputeBigramProb(acBackoff);
		fBackoff = LMProb.m_fLogBackOff;

		LMProb = ComputeBigramProb(acBigram);
		fBigram = LMProb.m_fLogProb;

		LMProb.m_fLogProb = fBigram + fBackoff;
		
		//sscanf(strBigram.c_str(), "%s_%s", strBuffer1, strBuffer2);

		//sprintf(strBuffer, "%s", pszToken);
		//LMProb = ComputeUnigramProb(strBuffer);

	}

	return (LMProb);
}

LM_PROB CHLabLM::ComputeBigramProb(std::string strBigram)
{
	/* Let's check whether we can find the trigram */
	LM_PROB LMProb;
	float fLike;
	float fBackoff, fBigram;
	
	char strBuffer[MAX_LINE_LEN];
	const char* pszToken = NULL;
	const char* pszToken1 = NULL;
	const char* pszToken2 = NULL;

	LMProb = m_hashBigram[strBigram];
	fLike  = LMProb.m_fLogProb;

	char acBackoff[MAX_LINE_LEN];
	char acBigram[MAX_LINE_LEN];

	char seps[]   = "_";

	/* Serach in the bigram */
	if (LMProb.m_fLogProb == 0)
	{
		std::string strTemp = strBigram;

		pszToken1 = strtok((char*)strTemp.c_str(), seps);
		pszToken2 = strtok(NULL, seps);

		
		//sscanf(strBigram.c_str(), "%s_%s", strBuffer1, strBuffer2);

		sprintf(acBackoff, "%s", pszToken1);

		sprintf(acBigram, "%s", pszToken2);

	
		LMProb = ComputeUnigramProb(acBackoff);
		fBackoff = LMProb.m_fLogBackOff;

		LMProb = ComputeUnigramProb(acBigram);
		fBigram = LMProb.m_fLogProb;

		LMProb.m_fLogProb = fBigram + fBackoff;
	}

	return (LMProb);
}

LM_PROB CHLabLM::ComputeUnigramProb(std::string strUnigram)
{
	LM_PROB LMProb;
	float fLike;

	LMProb = m_hashUnigram[strUnigram];

	fLike = LMProb.m_fLogProb;

	/* Serach in the bigram */
	if (LMProb.m_fLogProb == 0)
	{
		//E_WARN("Unigram for %s cannot be found. ", strUnigram.c_str());

		fLike = -10.0;
	}

	return (LMProb);

}

int CHLabLM::ReadLM(const char* szFile)
{
	FILE* pFile = NULL;

	pFile = fopen(szFile, "rt");

	float fLogProb;
	float fBackoffProb;
	char  acStr1[MAX_LINE_LEN];
	char  acStr2[MAX_LINE_LEN];
	char  acStr3[MAX_LINE_LEN];
	char  acStrEntire[MAX_LINE_LEN];
	std::string strEntire;

	LM_PROB stLMProb;

	char acLineBuffer[MAX_LINE_LEN];

	if (pFile == NULL)
	{
		E_FATAL("%s cannot be opened\n", szFile);
	}

	// skipping until we meet the 1-grams 
	while (fgets(acLineBuffer, MAX_LINE_LEN, pFile) != NULL)
	{
		if (strncmp(acLineBuffer, "\\1-grams:", strlen("\\1-grams:")) == 0)
		{
			break;
		}
	}

	fgets(acLineBuffer, MAX_LINE_LEN, pFile); // Skpping one blank-line

	while (fgets(acLineBuffer, MAX_LINE_LEN, pFile) != NULL)
	{
		if (strncmp(acLineBuffer, "\\2-grams:", strlen("\\2-grams:")) == 0)
		{
			break;
		}

		/* uni-gram line reading */

		//printf("%s", acLineBuffer);

		sscanf(acLineBuffer, "%f %s %f", &fLogProb, acStr1, &fBackoffProb);

		strEntire.clear();
		strEntire.replace(0, strlen(acStr1), acStr1);

		stLMProb.m_fLogProb    = fLogProb;
		stLMProb.m_fLogBackOff = fBackoffProb;

		m_hashUnigram.insert(std::make_pair(strEntire, stLMProb));
		
	}

	fgets(acLineBuffer, MAX_LINE_LEN, pFile); // Skpping one blank-line

	while (fgets(acLineBuffer, MAX_LINE_LEN, pFile) != NULL)
	{
		if (strncmp(acLineBuffer, "\\3-grams:", strlen("\\3-grams:")) == 0)
		{
			break;
		}

		/* bi-gram line reading */

		sscanf(acLineBuffer, "%f %s %s %f", &fLogProb, acStr1, acStr2, &fBackoffProb);

	
		sprintf(acStrEntire, "%s_%s", acStr1, acStr2);

		strEntire.clear();
		strEntire.replace(0, strlen(acStrEntire), acStrEntire);

		stLMProb.m_fLogProb    = fLogProb;
		stLMProb.m_fLogBackOff = fBackoffProb;

		m_hashBigram.insert(std::make_pair(strEntire, stLMProb));
	
	}

	fgets(acLineBuffer, MAX_LINE_LEN, pFile); // Skpping one blank-line

	while (fgets(acLineBuffer, MAX_LINE_LEN, pFile) != NULL)
	{

		if (strncmp(acLineBuffer, "\\end\\", strlen("\\end\\")) == 0)
		{
			break;
		}
	
		/* tri-gram line reading */

		

		sscanf(acLineBuffer, "%f %s %s %s %f", &fLogProb, acStr1, acStr2, acStr3, &fBackoffProb);

		fBackoffProb = 0.0;
		sprintf(acStrEntire, "%s_%s_%s", acStr1, acStr2, acStr3);

		strEntire.clear();
		strEntire.replace(0, strlen(acStrEntire), acStrEntire);


		stLMProb.m_fLogProb    = fLogProb;
		stLMProb.m_fLogBackOff = fBackoffProb;

	
		m_hashTrigram.insert(std::make_pair(strEntire, stLMProb));
	
		
	}

	


	return (0);
}