#include "libutil.h"

#include "HLabTranscription.h"

CHLabTranscription::CHLabTranscription(void) :
	m_pPhoneList(NULL), 
	m_pDict(NULL),
	m_iLen(0),
	m_pTransFile(NULL)
{
}

CHLabTranscription::~CHLabTranscription(void)
{
	if (m_pTransFile != NULL)
	{
		fclose (m_pTransFile);
	}
}

int CHLabTranscription::Init(const char* pszTransFileName)
{
	m_pTransFile = fopen(pszTransFileName, "rt");

	if (m_pTransFile == NULL)
	{
		E_FATAL("File open error");
	}
	
	return (0);
}

 
std::vector<int> CHLabTranscription::ReadTranscriptionFileLine(void)
{
	if (m_pPhoneList == NULL)
	{
		E_FATAL("[FATAL], invalid phone class pointer");
	}

	if (m_pDict == NULL)
	{
		E_FATAL("[FATAL], invalid number of dictionary words");
	}

	
	char  acBuffer[MAX_LINE];
	char  acWord[MAX_LINE];
	int   iLen;
	char* pszToken;
	char seps[]   = " ,\t\n\r";
	char* pszReturn;
	std::vector<int> vecWordPhoneSeq;
	std::vector<int> vecTransPhoneSeq;

	m_vecTranscriptionLine.clear();

	pszReturn = fgets(acBuffer, MAX_LINE, m_pTransFile);

	if (pszReturn != NULL)
	{
		pszToken = strtok(acBuffer, seps);

		if (strcmp(pszToken, "<s>") != 0)
		{
			fprintf(stderr, "[FATAL] The transcription should start with <s>\n");
			exit (0);
		}

		while (pszToken != NULL && strcmp(pszToken, "</s>") != 0)
		{
			pszToken = strtok(NULL, seps);

			if (pszToken != NULL)
			{
				vecWordPhoneSeq = m_pDict->GetDictWords(pszToken);
				m_vecTranscriptionLine.insert(m_vecTranscriptionLine.end() , vecWordPhoneSeq.begin(), vecWordPhoneSeq.end());
			}
		}
		
		m_iLen++;

	}

	return m_vecTranscriptionLine;
}

int CHLabTranscription::SetPhoneList(CHLabPhoneList* pPhoneList)
{
	if (pPhoneList == NULL)
	{
		fprintf(stderr, "[FATAL] null pointer\n");

		exit (0);
	}

	if (pPhoneList->GetNumPhones() <= 0)
	{
		fprintf(stderr, "[FATAL] invalid number of phones\n");

		exit (0);
	}

	m_pPhoneList = pPhoneList;

	return (0);
}

int CHLabTranscription::SetDictList(CHLabDict*      pDict)
{
	if (pDict == NULL)
	{
		fprintf(stderr, "[FATAL] null pointer\n");

		exit (0);
	}

	if (pDict->GetNumDictWords() <= 0)
	{
		fprintf(stderr, "[FATAL] invalid number of dictionary words\n");

		exit (0);
	}

	m_pDict = pDict;

	return (0);
}

#if 0

int CHLabTranscription::ReadTranscriptionFileLine()
{
	return (0);
}

#endif

int CHLabTranscription::ReadTranscriptionFile(const char* pszFile)
{
	FILE* pFile = NULL;

	pFile = fopen(pszFile, "rb");

	if (pFile == NULL)
	{
		fprintf(stderr, "[FATAL], File cannot be opened.\n");

		exit (0);
	}

	if (m_pPhoneList == NULL)
	{
		fprintf(stderr, "[FATAL], invalid phone class pointer");

		exit (0);
	}

	if (m_pDict == NULL)
	{
		fprintf(stderr, "[FATAL], invalid phone class pointer");

		exit (0);
	}


	char  acBuffer[MAX_LINE];
	char  acWord[MAX_LINE];
	int   iLen;
	char seps[]   = " ,\t\n\r";

	char* pszToken;
	std::vector<int> vecTransPhoneSeq;
	std::vector<int> vecWordPhoneSeq;
	int         iPhoneID;

	m_iLen = 0;

	printf("Reading the transcription file %s\n", pszFile);


	while (fgets(acBuffer, MAX_LINE, pFile) != NULL)
	{
		
		
		pszToken = strtok(acBuffer, seps);

		if (strcmp(pszToken, "<s>") != 0)
		{
			fprintf(stderr, "[FATAL] The transcription should start with <s>\n");

			exit (0);
		}

	
		vecTransPhoneSeq.clear();

		while (pszToken != NULL && strcmp(pszToken, "</s>") != 0)
		{
			pszToken = strtok(NULL, seps);

			if (pszToken != NULL)
			{
				vecWordPhoneSeq = m_pDict->GetDictWords(pszToken);

				vecTransPhoneSeq.insert(vecTransPhoneSeq.end() , vecWordPhoneSeq.begin(), vecWordPhoneSeq.end());
			}
		}

		
		
		m_iLen++;

		m_vecTranscription.push_back(vecTransPhoneSeq);
	}

	return (0);
}

int CHLabTranscription::ListTranscription()
{

	std::vector<std::vector<int> >::iterator itr; // m_mapTranscription;
	std::vector<int>::iterator itr2;

	
	for (itr = m_vecTranscription.begin(); itr != m_vecTranscription.end(); itr++)
	{

		for (itr2 = (*itr).begin(); itr2 != (*itr).end(); itr2++)
		{
			std::cout << (*itr2) << " ";
		}

		std::cout <<  std::endl;
	}

	return (0);
}
