#include <math.h>

#include "HLabViterbi.h"

#include "libutil.h"

#define MAX2(X, Y)       (((X) > (Y))? (X) : (Y))
#define MAX3(X, Y, Z)    MAX2(MAX2(X, Y), Z)

#define MIN2(X, Y) (((X) < (Y))? (X) : (Y))
#define MIN3(X, Y, Z) (MIN2(MIN2(X, Y), Z))

#define SQ(X) ((X) * (X))

/**
 * @file
 * @author  Chanwoo Kim
 * @version 1.0
 *
 * @section LICENSE
 *
 * 
 *
 * @section DESCRIPTION
 *
 * @version
 *   March, 29, 2010 Modify the code to handle the non-emitting state
 * 
 */


CHLabViterbi::CHLabViterbi(void) :
	m_pHMM(NULL),
	m_pFeat(NULL),
	m_iDim(39),
	m_iT(0),
	m_fOutNonEmitSt_L(-HLab_INFINITY),
	m_iOutNonEmitBackPointer(0)
{
	int i;

	for (i  = 0; i < MAX_NUM_STATES; i++)
	{
		m_afLikelihood[i] = -HLab_INFINITY;
	}

	for (i  = 0; i < MAX_NUM_STATES; i++)
	{
		m_aiBackpointer[i] = 0;
	}
}

CHLabViterbi::~CHLabViterbi(void)
{
	int i;

	/* This is very very inefficient */

	
}

int CHLabViterbi::Init(CHLabHMM* pHMM, CHLabFeat* pFeat)
{
		m_pHMM  = pHMM;
		m_pFeat = pFeat;
		m_iT    = 0;

		return (0);
}


float CHLabViterbi::Run(float* pfOutLikelihood, int* piOutBackPointer,
						float* pfHighestLikelihood,
						float fInLikelihood,    int iInBackPointer,
						float fTH)
{
	int j;
	float fLogPDF;
	float afLH[MAX_NUM_STATES];
	int   aiBP[MAX_NUM_STATES];
	int iState;

	float fVal1 = -HLab_INFINITY;
	float fVal2 = -HLab_INFINITY;
	float fVal3 = -HLab_INFINITY;
	float fValMAX;

	
	
	if (m_iT == 0)
	{
		afLH[0] =  GaussianPDF(m_iT, 0);

		afLH[1] = -HLab_INFINITY;
		afLH[2] = -HLab_INFINITY;
		afLH[3] = -HLab_INFINITY;
		afLH[4] = -HLab_INFINITY;

		aiBP[0] = 0;
		aiBP[1] = 0;
		aiBP[2] = 0;
		aiBP[3] = 0;
		aiBP[4] = 0;

	}
	else
	{
		if (fInLikelihood  <= fTH &&
			m_afLikelihood[0] <= fTH)
		{
			afLH[0] = -HLab_INFINITY;
			aiBP[0] = -1;
		}
		else
		{
			fVal1 = fInLikelihood;
			fVal2 = m_afLikelihood[0] + this->m_pHMM->m_ppfTrans[0][0];

			fValMAX = MAX2(fVal1, fVal2);

			if (fValMAX == fVal1)
			{
				aiBP[0] = iInBackPointer;
			}
			else if(fValMAX == fVal2)
			{
				aiBP[0] = m_aiBackpointer[0];
			}
			else
			{
				E_FATAL("Impossible back pointer");
			}

			afLH[0] = fValMAX +  GaussianPDF(m_iT, 0);

		}

		iState = 1;
		
		if (m_afLikelihood[0] <= fTH &&
			m_afLikelihood[1] <= fTH)
		{
			afLH[iState] = -HLab_INFINITY;

			aiBP[iState] = -1;
		}
		else
		{
			fVal2 = m_afLikelihood[iState - 1] + this->m_pHMM->m_ppfTrans[iState - 1][iState];
			fVal3 = m_afLikelihood[iState]     + this->m_pHMM->m_ppfTrans[iState    ][iState];
		
			fValMAX = MAX2(fVal2, fVal3);

			if(fValMAX == fVal2)
			{
				aiBP[iState] = m_aiBackpointer[iState - 1];
			}
			else if (fValMAX == fVal3)
			{
				aiBP[iState] = m_aiBackpointer[iState];
			}
			else
			{
				E_FATAL("Impossible back pointer");
			}

			afLH[iState] = fValMAX + GaussianPDF(m_iT, iState);

		}

		

		for (iState = 2; iState < this->m_pHMM->m_iNumStates; iState++)
		{
			if (m_afLikelihood[iState - 2] <= fTH &&
				m_afLikelihood[iState - 1] <= fTH &&
				m_afLikelihood[iState]     <= fTH)
			{
				afLH[iState] = -HLab_INFINITY; /* pruned */

				aiBP[iState] = -1;
			}
			else
			{
				fVal1 = m_afLikelihood[iState - 2] + this->m_pHMM->m_ppfTrans[iState - 2][iState];
				fVal2 = m_afLikelihood[iState - 1] + this->m_pHMM->m_ppfTrans[iState - 1][iState];
				fVal3 = m_afLikelihood[iState]     + this->m_pHMM->m_ppfTrans[iState    ][iState];
				fValMAX = MAX3(fVal1, fVal2, fVal3);
				
				afLH[iState] = fValMAX + GaussianPDF(m_iT, iState);

				if (fValMAX == fVal1)
				{
					aiBP[iState] = m_aiBackpointer[iState - 2];
				}
				else if(fValMAX == fVal2)
				{
					aiBP[iState] = m_aiBackpointer[iState - 1];
				}
				else if (fValMAX == fVal3)
				{
					aiBP[iState] = m_aiBackpointer[iState];
				}
				else
				{
					E_FATAL("Impossible back pointer");
				}
			}
		}

		/* Non-emitting state case */
		iState = this->m_pHMM->m_iNumStates;
		if (afLH[iState - 2] <= fTH && afLH[iState - 1] <= fTH)
		{
			m_fOutNonEmitSt_L = -HLab_INFINITY;

			m_iOutNonEmitBackPointer = -1;
		}
		else
		{
			fVal1 = afLH[iState - 2] + this->m_pHMM->m_ppfTrans[iState - 2][iState];
			fVal2 = afLH[iState - 1] + this->m_pHMM->m_ppfTrans[iState - 1][iState];
				
			m_fOutNonEmitSt_L = MAX2(fVal1, fVal2);
							

			if (m_fOutNonEmitSt_L == fVal1)
			{
					m_iOutNonEmitBackPointer = aiBP[iState - 2];
			}
			else if(m_fOutNonEmitSt_L == fVal2)
			{
					m_iOutNonEmitBackPointer = aiBP[iState - 1];
			}

		
		}

	}

	if (pfOutLikelihood != NULL)
	{
		*pfOutLikelihood = m_fOutNonEmitSt_L;
	}

	if (piOutBackPointer != NULL)
	{
		*piOutBackPointer = m_iOutNonEmitBackPointer;
	}

	if (pfHighestLikelihood != NULL)
	{
		int i;
		float fHighest = -HLab_INFINITY;

		for (i = 0; i < this->m_pHMM->m_iNumStates; i++)
		{
			if (afLH[i] >  fHighest)
			{
				fHighest = afLH[i];
			}
		}

		*pfHighestLikelihood = MAX2(fHighest, m_fOutNonEmitSt_L);
		
	}

	memcpy(m_afLikelihood,  afLH, sizeof(float) * MAX_NUM_STATES);
	memcpy(m_aiBackpointer, aiBP, sizeof(int)   * MAX_NUM_STATES);

	

	m_iT++;
	return (0.0);
}





float CHLabViterbi::GaussianPDF(int iTestFrameIndex, int iStateIndex)
{
	float fLogPDF;
	int i;
	float fVar;
	float fMean;
	float f_x;

	fLogPDF = 0.0;

	for (i = 0; i < m_iDim; i++)
	{
		fMean = m_pHMM->m_ppfMean[iStateIndex][i];
		fVar  = m_pHMM->m_ppfVar[iStateIndex][i];
		f_x   = m_pFeat->m_ppfFeat[iTestFrameIndex][i];

		fLogPDF -= (log(sqrt(fVar)) +  SQ(f_x - fMean) / (2.0 * fVar) + 1);
		
	}

	return (fLogPDF);
}


