#pragma once

#ifdef WIN32
	#if defined _USRDLL 
		#define DllExport _declspec(dllexport)
	#else
		#define DllExport _declspec(dllimport)
	#endif 
#endif

#include "HLabHMM.h"
#include "HLabFeat.h"
#include "HLabConst.h"

/**
* @file
* @author  Chanwoo Kim (chanwook@cs.cmu.edu)
* @version 1.0
*
* @section LICENSE
*
* @section DESCRIPTION
*
* The CHLabViterbiAlignment class is for forced alignment
* Viterbi processing is implemented in CHLabViterbi, but with that class,
* We cannot do state level backtracking.
*
*
*/

class DllExport CHLabViterbi
{
	const CHLabHMM*  m_pHMM;
	const CHLabFeat* m_pFeat;
	int m_iDim;
	
	float m_afLikelihood[MAX_NUM_STATES];
	int   m_aiBackpointer[MAX_NUM_STATES];
	
	int m_iT;
	float            m_fOutNonEmitSt_L;
	int              m_iOutNonEmitBackPointer;

	float GaussianPDF(int iTestFrameIndex, int iStateIndex);
public:
	CHLabViterbi(void);
	virtual ~CHLabViterbi(void);

	int Init(CHLabHMM* pHMM, CHLabFeat* pFeat);
	
	float Run(float* pfOutLikelihood, int* piOutBackPointer,
			  float* pfHighestLikelihood,
 		     float fInLikelihood,    int iInBackPointer, float fTH = -HLab_INFINITY);

	inline float GetNonEmissionStateProb(void)
	{
		return m_fOutNonEmitSt_L;
	}

	inline int GetNonEmissionStateBackpointer(void)
	{
		return m_iOutNonEmitBackPointer;
	}
 	
};
