#include "HLabViterbiAlignment.h"

#include "libutil.h"

CHLabViterbiAlignment::CHLabViterbiAlignment(void)
	: m_pppfLikelihoodTable(NULL),
	  m_ppiStateBoundary(NULL),
	  m_iNumHMMs(0),
	  m_iFeatLen(0),
	  m_iMaxNumStates(0)
{
}

CHLabViterbiAlignment::~CHLabViterbiAlignment(void)
{
	
}

int CHLabViterbiAlignment::Init(CHLabHMM* pHMM, 
							    int iNumHMM,
			                    CHLabFeat* pFeat,
			                    float fTH)
{
	int i, j;
	int iNumStates;

	if (pHMM == NULL)
	{
		E_FATAL("Invalid HMM pointer\n");
	}

	if (iNumHMM <= 0)
	{
		E_FATAL("Invalid number of HMMs\n");
	}

	if (pFeat == NULL)
	{
		E_FATAL("Invalid featuer pointer\n");
	}

	m_iFeatLen = pFeat->GetLen();
	m_iNumHMMs = iNumHMM;
	m_fTH      = fTH;
	m_pHMM     = pHMM;
	m_pFeat    = pFeat;
	m_iDim     = pFeat->GetFeatDim();

		

	m_iMaxNumStates = 0;


	for (i = 0; i < m_iNumHMMs; i++)
	{
		iNumStates = m_pHMM[i].GetNumStates();

		if (iNumStates <= 0)
		{
			E_FATAL("Invalid number of HMM states");
		}

		if (iNumStates > m_iMaxNumStates)
		{
			m_iMaxNumStates = iNumStates;
		}
	}

	
	/* Using number of states + 1 to handle the final non-emitting states */
	m_vec2d_stateStartBounday.resize(m_iNumHMMs, m_iMaxNumStates + 1);
	m_vec2d_stateStartBounday.SetVal(-1);

	m_vec2d_stateEndBoundary.resize(m_iNumHMMs, m_iMaxNumStates + 1);
	m_vec2d_stateEndBoundary.SetVal(-1);
	
	m_vec2d_likelihoodTable.resize(m_iNumHMMs , m_iMaxNumStates + 1);
	m_vec2d_likelihoodTable.SetVal(-HLab_INFINITY);

	m_vec2d_likelihoodTableTemp.resize(m_iNumHMMs , m_iMaxNumStates + 1);
	m_vec2d_likelihoodTableTemp.SetVal(-HLab_INFINITY);

	m_vec3d_backtrackTable.resize(m_iNumHMMs , m_iFeatLen, m_iMaxNumStates + 1);
	m_vec3d_backtrackTable.SetVal(-1);
	
	return (0);
}


int CHLabViterbiAlignment::Run(float* pLikelihood)
{
	float fLogPDF;
	int   iLen = m_pFeat->GetLen();
	int   iHMMIndex;
	int   iT;
	int   iState;

	float fVal0, fVal1, fVal2;
	float fValMAX;
	float fNonemitting;

	printf("%d\n", iLen);

	fNonemitting = 0.0;

	

	

	/* Time synchronus */
	for (iT = 0; iT < m_pFeat->GetLen(); iT++)
	{

		m_vec2d_likelihoodTableTemp.SetVal(-HLab_INFINITY);

		/* Likelihood table calculation */

		/* Fix me! HMM topology is hard-coded */

		
		

		if (iT == 0)
		{
			m_vec2d_likelihoodTable[0][0] =GaussianPDF(0, 0, 0);
			m_vec3d_backtrackTable[0][0][0] = 1;

			continue;
		}

		
		fNonemitting = -HLab_INFINITY;

		for (iHMMIndex = 0; iHMMIndex < m_iNumHMMs; iHMMIndex++)
		{

			/* The first state is considered to be a special case 
			There are only two ways entering this state  (due to the non-emitting state) */

			iState = 0;
			{
				if (iHMMIndex >= 1)
				{
					fVal1 = m_vec2d_likelihoodTable[iHMMIndex - 1][m_pHMM[0].GetNumStates()];
				}
				else
				{
					fVal1 = -HLab_INFINITY;
				}
				
				fVal0 = m_vec2d_likelihoodTable[iHMMIndex][iState];

				if (fVal1 <= m_fTH && fVal0 <= m_fTH)
				{
					m_vec2d_likelihoodTableTemp[iHMMIndex][iState] = -HLab_INFINITY;
					m_vec3d_backtrackTable[iHMMIndex][iT][iState]  = -1;
				}
				else
				{
					//fVal1  = fNonemitting;
					fVal0 += m_pHMM[iHMMIndex].m_ppfTrans[iState][iState];

					fValMAX = MAX2(fVal1, fVal0);
					
					m_vec2d_likelihoodTableTemp[iHMMIndex][iState]=  
						fValMAX + GaussianPDF(iT, iHMMIndex, iState);

					if (fValMAX == fVal0)
					{
						m_vec3d_backtrackTable[iHMMIndex][iT][iState]  = 0;
					}
					else if (fValMAX == fVal1)
					{
						m_vec3d_backtrackTable[iHMMIndex][iT][iState]  = 1;
					}
				}
			}
			

			for (iState = 1; iState < m_pHMM[0].GetNumStates(); iState++)
			{
				if (iState >= 2)
				{
					fVal2 = m_vec2d_likelihoodTable[iHMMIndex][iState - 2];
				}
				else
				{
#if 0
					if (iHMMIndex >= 1)
					{
						fVal2 = m_vec2d_likelihoodTable[iHMMIndex - 1][5];
					}
					else
#endif
					{
						fVal2 = -HLab_INFINITY;
					}
				}

				fVal1 = m_vec2d_likelihoodTable[iHMMIndex][iState - 1];
				fVal0 = m_vec2d_likelihoodTable[iHMMIndex][iState];

				if (fVal0 <= m_fTH && fVal1 <= m_fTH && fVal2 <= m_fTH)
				{
					m_vec2d_likelihoodTableTemp[iHMMIndex][iState] = -HLab_INFINITY; /* pruned */
					m_vec3d_backtrackTable[iHMMIndex][iT][iState]  = -1;
				}
				else
				{
					/* If it is from the non-emitting state, there's no transition prob. */
					if (iState >= 2)
					{
						fVal2 += m_pHMM[iHMMIndex].m_ppfTrans[iState - 2][iState];
					}
					else
					{
						fVal2 = -HLab_INFINITY;
					}

					fVal1 += m_pHMM[iHMMIndex].m_ppfTrans[iState - 1][iState];
					fVal0 += m_pHMM[iHMMIndex].m_ppfTrans[iState    ][iState];

					fValMAX = MAX3(fVal2, fVal1, fVal0);
					
					m_vec2d_likelihoodTableTemp[iHMMIndex][iState] =
						fValMAX + GaussianPDF(iT, iHMMIndex, iState);

					if      (fValMAX == fVal0)
					{
						m_vec3d_backtrackTable[iHMMIndex][iT][iState] = 0;
					}
					else if (fValMAX == fVal1)
					{
						m_vec3d_backtrackTable[iHMMIndex][iT][iState] = 1;
					}
					else if (fValMAX == fVal2)
					{
						m_vec3d_backtrackTable[iHMMIndex][iT][iState] = 2;
					}
					else
					{
						E_WARN("Impossible transition. something is wrong");

						exit (0);

						return (-1);
					}
				}
			}

			/* Non-emitting state case */
			iState = m_pHMM[0].GetNumStates();
			{
				fVal2 = m_vec2d_likelihoodTableTemp[iHMMIndex][iState - 2];
				fVal1 = m_vec2d_likelihoodTableTemp[iHMMIndex][iState - 1];
				
				if (fVal2 <= m_fTH && fVal1 <= m_fTH)
				{
					m_vec2d_likelihoodTableTemp[iHMMIndex][iState] = -HLab_INFINITY;
					m_vec3d_backtrackTable[iHMMIndex][iT][iState]  = -1;

					fValMAX = -HLab_INFINITY;
				}
				else
				{
					fVal2 += m_pHMM[iHMMIndex].m_ppfTrans[iState - 2][iState];
					fVal1 += m_pHMM[iHMMIndex].m_ppfTrans[iState - 1][iState];
				

					fValMAX = MAX2(fVal2, fVal1);
						
					m_vec2d_likelihoodTableTemp[iHMMIndex][iState]= fValMAX;

					if (fValMAX == fVal1)
					{
							
							m_vec3d_backtrackTable[iHMMIndex][iT][iState]  = 1;
					}
					else if(fValMAX == fVal2)
					{
							m_vec3d_backtrackTable[iHMMIndex][iT][iState]  = 2;
					}
				}

				fNonemitting = fValMAX;

			
			}


		} // for (iHMMIndex = 0; iHMMIndex < m_iNumHMMs; iHMMIndex++)
	

		for (iHMMIndex = 0; iHMMIndex < m_iNumHMMs; iHMMIndex++)
		{
			for (iState = 0; iState <= m_pHMM[0].GetNumStates(); iState++)
			{
				m_vec2d_likelihoodTable[iHMMIndex][iState] = m_vec2d_likelihoodTableTemp[iHMMIndex][iState];

			
				
			}
		}

		

	} // for (iT = 0; iT < m_pFeat->GetLen(); iT++)



	printf("[FINAL LIKELIHOOD] %f\n", m_vec2d_likelihoodTable[m_iNumHMMs - 1][m_iMaxNumStates]);


	


	

	

	/* Backtracking */
	{
		int iNewState;
		iHMMIndex = m_iNumHMMs - 1;
		iT = m_pFeat->GetLen() - 1;
		iState    = m_pHMM[iHMMIndex].GetNumStates();
		int iPointer;


		m_vec2d_stateEndBoundary[iHMMIndex][iState] = iT;

		while (iT > 0)
		{
			
			
			iPointer = m_vec3d_backtrackTable[iHMMIndex][iT][iState];

	

			if (iPointer == 0)
			{
				iT--;
			}
			else if (iPointer == 1)
			{
				m_vec2d_stateStartBounday[iHMMIndex][iState] = iT;


				if (iState != m_pHMM[iHMMIndex].GetNumStates())
					iT--;
				
				if (iState >= 1)
				{
					/* Mark transition boundary */
					iState--;
					m_vec2d_stateEndBoundary[iHMMIndex][iState] = iT;
				}
				else
				{
					iHMMIndex--;

					iState = m_pHMM[iHMMIndex].GetNumStates();
					m_vec2d_stateEndBoundary[iHMMIndex][iState] = iT;
				}
			}
			else if (iPointer == 2)
			{
				m_vec2d_stateStartBounday[iHMMIndex][iState] = iT;

				

				if (iState != m_pHMM[iHMMIndex].GetNumStates())
					iT--;
				

				if (iState >= 2)
				{
					iState -= 2;
					m_vec2d_stateEndBoundary[iHMMIndex][iState] = iT;
				}
				else if (iState == 1)
				{
					iHMMIndex--;

					iState = m_pHMM[iHMMIndex].GetNumStates();

					m_vec2d_stateEndBoundary[iHMMIndex][iState] = iT;
				}
				else
				{
					E_WARN("Impossible transition. something is wrong");

					exit (0);

					return (-1);
				}
	
			}
			else
			{
				E_WARN("Failed to reach the origin");

				exit (0);

				return (-1);
			}

			
		}

		

		/* Check whether it is first HMM first or secnd HMM state */

	}

	m_vec2d_stateStartBounday[0][0] = 0;

	

	if (iHMMIndex != 0)
	{

		E_WARN("Failed to reach the origin");

		return (-1);
	}

	return (0);
}



float CHLabViterbiAlignment::GaussianPDF(int iTestFrameIndex, 
										 int iHMMIndex,
										 int iStateIndex)
{
	float fLogPDF;
	int i;
	float fVar;
	float fMean;
	float f_x;

	fLogPDF = 0.0;

	for (i = 0; i < m_iDim; i++)
	{
		fMean = m_pHMM[iHMMIndex].m_ppfMean[iStateIndex][i];
		fVar  = m_pHMM[iHMMIndex].m_ppfVar[iStateIndex][i];

		f_x   = m_pFeat->m_ppfFeat[iTestFrameIndex][i];

		fLogPDF -= ((log(sqrt(fVar)) + SQ(f_x - fMean) / (2.0 * fVar)) + 1);
		
	}

	return (fLogPDF);
}

