#pragma once

#ifdef WIN32
	#if defined _USRDLL 
		#define DllExport _declspec(dllexport)
	#else
		#define DllExport _declspec(dllimport)
	#endif 
#endif

#include "HLabHMM.h"
#include "HLabFeat.h"
#include "HLabConst.h"
#include "HLabVector.h"


/**
* @file
* @author  Chanwoo Kim (chanwook@cs.cmu.edu)
* @version 1.0
*
* @section LICENSE
*
* @section DESCRIPTION
*
* The CHLabViterbiAlignment class is for forced alignment
* Viterbi processing is implemented in CHLabViterbi, but with that class,
* We cannot do state level backtracking.
*
*
*/

class DllExport CHLabViterbiAlignment
{
	/**
		m_pppfLikelihoodTable[HMMIndex][FeatIndex][StateIndex]

		Number of states + 1 (due to the non-emitting state)
	*/
	float***  m_pppfLikelihoodTable;
	vector2d<float> m_vec2d_likelihoodTable;
	vector2d<float> m_vec2d_likelihoodTableTemp;

	/**

		2 is the tranition from the current minus two states
		1 is the tranition from the current minus one states
		0 is the tranition from the current state

	*/
	vector3d<float> m_vec3d_backtrackTable;
	int       m_iNumHMMs;
	int       m_iFeatLen;
	CHLabHMM* m_pHMM;
	CHLabFeat* m_pFeat;
	int        m_iDim;
	int        m_iMaxNumStates;
	
	int**   m_ppiStateBoundary;
	vector2d<int> m_vec2d_stateStartBounday;
	vector2d<int> m_vec2d_stateEndBoundary;
	float   m_fTH;

	float GaussianPDF(int iTestFrameIndex, 
					  int iHMMIndex,
				      int iStateIndex);

public:
	CHLabViterbiAlignment(void);
	virtual ~CHLabViterbiAlignment(void);

	/**
      * Init
      * 
      * @param 
      * @return 
	  *
      */

	int Init(CHLabHMM* pHMM, 
		     int iNumHMM,
			 CHLabFeat* pFeat,
			 float fTH = -HLab_INFINITY);
	
	 /**
      * Run
      * 
      * @param 
      * @return 
	  *
      */
	int Run(float* pLikelihood);


	
	/**
      * GetStateStartBoundary
      * 
      * @param 
      * @return 
	  *
      */
	inline vector2d<int>& GetStateStartBoundary(void)
	{
		return (m_vec2d_stateStartBounday);
	}

	/**
      * GetStateEndBoundary
      * 
      * @param 
      * @return 
	  *
      */
	inline vector2d<int>& GetStateEndBoundary(void)
	{
		return (m_vec2d_stateEndBoundary);
	}
};
