#include "HLabWordViterbi.h"

#include "libutil.h"

CHLabWordViterbi::CHLabWordViterbi(void) :
	m_pHMM(NULL), 
	m_pFeat(NULL),
	m_pViterbi(NULL),
	m_iNumHMMs(0)
{
}

CHLabWordViterbi::~CHLabWordViterbi(void)
{
	if (m_pViterbi != NULL)
	{
		delete [] m_pViterbi;
	}
}


int CHLabWordViterbi::Init(const CHLabHMM* pHMM, 
						   int iNumHMMs,
			               std::vector<int> vecHMMIndex,
		                   const CHLabFeat* pFeat)
{
	m_pHMM        = pHMM;
	m_iNumHMMs    = iNumHMMs;
	m_pFeat       = pFeat;
	m_vecHMMIndex = vecHMMIndex;

	int i;

	

	m_pViterbi = new CHLabViterbi [m_vecHMMIndex.size()];

	if (m_pViterbi == NULL)
	{
		E_FATAL("Memory allocation failure!");
	}

	for (i = 0; i < m_vecHMMIndex.size(); i++)
	{
		m_pViterbi[i].Init((CHLabHMM*) &m_pHMM[vecHMMIndex[i]], (CHLabFeat*) m_pFeat);
	}

	return (0);
}


float CHLabWordViterbi::Run(float* pfOutLikelihood, 
							int* piOutBackPointer,
							float* pfHighestLikelihood,
 							float fInLikelihood,    
							int iInBackPointer, 
							float fTh)
{
	int i;
	float fHighest;
	float fOutTemp;
	float fTemp;
	float fTempHighest;
	int   iOutBackPointer;
	int   iTempInBackPointer;

	/* hard-wiring .. fix me! */
	float afOutLikelihood[100];
	int   aiOutBackPointer[100];


	afOutLikelihood[0]  = fInLikelihood;
	aiOutBackPointer[0] = iInBackPointer ;
	

	for (i = 0; i < m_vecHMMIndex.size(); i++)
	{
		afOutLikelihood[i + 1]  = m_pViterbi[i].GetNonEmissionStateProb();
		aiOutBackPointer[i + 1] = m_pViterbi[i].GetNonEmissionStateBackpointer();
	}
	

	*pfOutLikelihood = -HLab_INFINITY;
	fTemp            = -HLab_INFINITY;
	fTempHighest     = -HLab_INFINITY;

	iOutBackPointer = 0;
	for (i = 0; i < m_vecHMMIndex.size(); i++)
	{
	

		m_pViterbi[i].Run(&fOutTemp,           // out likelihood
						  &iOutBackPointer,    // output back pointer
					      &fTemp,              // highest likelihood
					      afOutLikelihood[i], 
						  aiOutBackPointer[i], 
						  fTh);

//		printf("[%d %d %f %d] " , i, iOutBackPointer, afOutLikelihood[i], aiOutBackPointer[i]);


		if (i == m_vecHMMIndex.size() - 1)
		{
			*piOutBackPointer = iOutBackPointer;
			
		}

		if (fTemp > fTempHighest)
		{
			fTempHighest = fTemp;
		}
	}


	*pfHighestLikelihood = fTempHighest;
	*pfOutLikelihood     = m_pViterbi[m_vecHMMIndex.size() - 1].GetNonEmissionStateProb();
	*piOutBackPointer    = m_pViterbi[m_vecHMMIndex.size() - 1].GetNonEmissionStateBackpointer();

	

	return (0.0);
}