#pragma once

#include <vector>

#ifdef WIN32
	#if defined _USRDLL 
		#define DllExport _declspec(dllexport)
	#else
		#define DllExport _declspec(dllimport)
	#endif 
#else
	#define DllExport 
#endif

/**
 * @file
 * @author  Chanwoo Kim (chanwook@cs.cmu.edu)
 * @version 1.0
 *
 * @section LICENSE
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details at
 * http://www.gnu.org/copyleft/gpl.html
 *
 * @section DESCRIPTION
 *
 * 
 */

template <typename T>
class DllExport vector2d
{
private:

 std::vector< std::vector<T> > m_data;

public:
 vector2d() {};

 vector2d(int i_x1, int i_x2)
 {
    for (int i = 0; i < i_x1; i++)
    {
       m_data.push_back(std::vector<T> (i_x2));
    }
 }

	inline       std::vector<T> & operator[] (int i) { return m_data[i]; }
	inline const std::vector<T> & operator[] (int i) const { return m_data[i]; }


	 void resize(int i_x1, int i_x2)
	 {
		m_data.resize(i_x1);

		for (int i = 0; i < i_x1; i++)
		{
		   m_data[i].resize(i_x2);
		}
	 }
};


template <typename T>
class DllExport vector3d
{
	private:

	std::vector< vector2d<T> > m_data;

	public:
	vector3d() {};

	vector3d(int i_x1, int i_x2, int i_x3)
	{
		for (int i = 0; i < i_x1; i++)
		{
		   m_data.push_back(vector2d<T> (i_x2, i_x3));
		}
	}

	inline       vector2d<T> & operator[] (int i)	     { return m_data[i]; }
	inline const vector2d<T> & operator[] (int i) const { return m_data[i]; }


	void resize(int i_x1, int i_x2, int i_x3)
	{
		int i, j;
		m_data.resize(i_x1);

		for (i = 0; i < i_x1; i++)
		{
		   m_data[i].resize(i_x2);

		   for (j = 0; j < i_x2; j++)
		   {
			   m_data[i][j].resize(i_x3);
		   }
		}
	}
};
