#include "HLabUnif.h"
#include "HLabConst.h"

CHLabUnif::CHLabUnif(void)  :
	m_pHMM(NULL),
	m_pAccum(NULL)	
{
}

CHLabUnif::~CHLabUnif(void)
{
	if (m_pHMM == NULL)
	{
		delete [] m_pHMM;
	}

	if (m_pAccum != NULL)
	{
		delete [] m_pAccum;
	}
}

int CHLabUnif::ParseCommandLine(int argc, char** argv)
{
	m_compMVConfig.ParseCommandLine(argc, argv);

	return (0);
}

int CHLabUnif::Init(void)
{
	
    //Reading the phone list file
	m_phoneList.ReadPhoneFile(m_compMVConfig.GetPhoneFile());
//	m_phoneList.ListPhones();

	//Reading the dictionary file
	m_dict.SetPhoneList(&m_phoneList);
	m_dict.ReadDictFile(m_compMVConfig.GetDictFile());
//	m_dict.ListDicWords();
	

	m_pHMM = new CHLabHMM [m_phoneList.GetNumPhones()];

	if (m_pHMM == NULL)
	{
		E_FATAL("Memoray allocation failure\n");
	}

	int i;
	char acBuffer[255];

	m_pAccum = new CHLabAccum [m_phoneList.GetNumPhones()];



	if (m_pAccum == NULL)
	{
		E_FATAL("Memoray allocation failure\n");
	}

	for (i = 0; i < m_phoneList.GetNumPhones(); i++)
	{
		sprintf(acBuffer, "%d", i);
		m_pHMM[i].Init(39, m_compMVConfig.GetNumStatesPerHMM(), acBuffer);

		m_pAccum[i].Init(39, m_compMVConfig.GetNumStatesPerHMM(), acBuffer);
	}
	
	m_control.ReadControlFile(m_compMVConfig.GetControlFile());
	//m_control.ListControl();
	
	m_transcription.SetPhoneList(&m_phoneList);
	m_transcription.SetDictList(&m_dict);
//	m_transcription.ReadTranscriptionFile(m_compMVConfig.GetTranscriptionFile());

	

	

	m_transcription.Init(m_compMVConfig.GetTranscriptionFile());
	//m_transcription.ListTranscription();

	
	return (0);
}

// Processing the control file line by line
int CHLabUnif::Run(void)
{
	std::vector<std::string>::iterator itr;
	char acFileName[MAX_LINE];
	FILE* pFile = NULL;

	CHLabFeat feat;

	int iIndex = 0;
	int iRefIndex = 0;
	int n, i, j, k;
	
	
	float afSum[MAX_FEAT_DIM];
	float afSqSum[MAX_FEAT_DIM];
	float afVar[MAX_FEAT_DIM];
	float afMean[MAX_FEAT_DIM];
	int   aiStateCount[MAX_NUM_STATES];
	int  iFeatLen;
	int  iFeatDim;
	int  iStateIndex;
	int  iAccFeatLen;
	int  iAccNumStates;
	float fTrans;

	std::vector<int> vecPhoneSeq;
	std::vector<int>::iterator itrPhoneSeq;

	iFeatLen = m_compMVConfig.GetFeatLen();

	if (iFeatLen <= 0)
	{
		E_FATAL("Invalid feature vector length");
	}

	// Initialization needs to be done

	iAccFeatLen   = 0;
	iAccNumStates = 0;

	std::string     phoneStr;


	
	memset(afSum,   0, sizeof(float) * MAX_FEAT_DIM);
	memset(afSqSum, 0, sizeof(float) * MAX_FEAT_DIM);
	memset(afVar,   0, sizeof(float) * MAX_FEAT_DIM);
	memset(afMean,  0, sizeof(float) * MAX_FEAT_DIM);

	float fStart, fEnd,fDur;
	int   iSt;
	int   iFr;

	
	// Read the control file line by line
	for (itr = m_control.m_vecControl.begin(); itr != m_control.m_vecControl.end(); itr++)
	{
		sprintf(acFileName, "%s/%s.%s", m_compMVConfig.GetFeatDir(),
			    (*itr).c_str(), m_compMVConfig.GetFeatExt());

		printf("%s\n", acFileName);
		
		feat.ReadFeatFile(acFileName, iFeatLen); 

		vecPhoneSeq = m_transcription.ReadTranscriptionFileLine();

		

		fStart = 0.0;
		fEnd   = 0.0;
		fDur   = (float) feat.GetLen() / (float)(vecPhoneSeq.size() * m_pHMM[0].GetNumStates());

		fEnd = fDur;


		for (itrPhoneSeq = vecPhoneSeq.begin(); itrPhoneSeq < vecPhoneSeq.end(); itrPhoneSeq++)
		{

			

			phoneStr = m_phoneList.GetPhoneSymbol(*itrPhoneSeq);
							

			iAccNumStates += m_pHMM[*itrPhoneSeq].GetNumStates();

			

			for (iSt = 0; iSt < m_pHMM[0].GetNumStates(); iSt++)
			{

				std::cout << phoneStr;
				printf("-%d [%d, %d] ", iSt, (int)(fStart + 0.5), (int) fEnd);

				fStart += fDur;
				fEnd   += fDur;
				fEnd   = MIN2(fEnd, feat.GetLen() - 1);

				for (iFr = (int)(fStart + 0.5); iFr <= fEnd; ++iFr)
				{
					m_pAccum[*itrPhoneSeq].m_piCount[iSt]++;

					for (k = 0; k < feat.GetFeatDim(); k++)
					{
						m_pAccum[*itrPhoneSeq].m_ppfSum[iSt][k]   += feat.m_ppfFeat[iFr][k];
						m_pAccum[*itrPhoneSeq].m_ppfSqSum[iSt][k] += SQ(feat.m_ppfFeat[iFr][k]);
					}
				}
			}

			
		}

		
		

		iAccFeatLen += feat.GetLen();
		iFeatDim     = feat.GetFeatDim();


		printf("%d %d %d\n", feat.GetLen(), feat.GetCepDim(), feat.GetFeatDim());

		for (i = 0; i < feat.GetLen(); i++)
		{
			for (j = 0; j < feat.GetFeatDim(); j++)
			{
				afSum[j]   += feat.m_ppfFeat[i][j];
				afSqSum[j] += SQ(feat.m_ppfFeat[i][j]);
			}
		}
	}

	E_INFO("Total number of states : %d\n", iAccNumStates);
	E_INFO("Total number of frames : %d\n", iAccFeatLen);

	if (iAccNumStates <= 0)
	{
		E_FATAL("Invalid number of states");
	}

	if (iAccFeatLen <= 0)
	{
		E_FATAL("Invalid number of feature length");
	}

	for (j = 0; j < feat.GetFeatDim(); j++)
	{
		afMean[j] = afSum[j] / (float)iAccFeatLen;
		afVar[j]  = afSqSum[j] / (float)iAccFeatLen - SQ(afMean[j]);
	}

	fTrans = (float)iAccNumStates / (float)iAccFeatLen;

	
	{
		float fMean, fVar;
		float fToCurrent, fToNext, fToNextNext;

		/* mean variance and transition prob. */
		for (i = 0; i < m_phoneList.GetNumPhones(); i++)
		{
			for (j = 0; j < m_pHMM[i].GetNumStates(); j++)
			{
				for (k = 0; k < feat.GetFeatDim(); k++)
				{
					m_pAccum[i].MeanVar(&fMean, &fVar, j, k);

					m_pHMM[i].SetMean(fMean, j, k);
					m_pHMM[i].SetVar(fVar, j, k);
				}
			}

			for (j = 0; j <= m_pHMM[i].GetNumStates(); j++)
			{
				for (k = 0; k <=  m_pHMM[i].GetNumStates(); k++)
				{
					m_pHMM[i].SetTrans(0, j, k);
				}
			}

			/* In case of uniform segmentation, for transition prob. let's just use the default values */

			m_pHMM[i].SetDefaultTrans();

			m_pHMM[i].WriteHMM(this->m_compMVConfig.GetModelDir());
		}
	}

	return (0);
}
