#include "HLabViterbiRecog.h"
#include "HLabConst.h"

#define SQ(X) ((X) * (X))

#define MAX2(X, Y) (((X) > (Y))? (X) : (Y))
#define MAX3(X, Y, Z) (MAX2(MAX2(X, Y), Z))

#define MIN2(X, Y) (((X) < (Y))? (X) : (Y))
#define MIN3(X, Y, Z) (MIN2(MIN2(X, Y), Z))

CHLabViterbiRecog::CHLabViterbiRecog(void)  :
	m_pHMM(NULL),
	m_pViterbi(NULL),
	m_pWordViterbi(NULL)
{
}

CHLabViterbiRecog::~CHLabViterbiRecog(void)
{
	if (m_pHMM == NULL)
	{
		delete [] m_pHMM;
	}

	if (m_pViterbi == NULL)
	{
		delete [] m_pViterbi;
	}

	if (m_pWordViterbi == NULL)
	{
		delete [] m_pWordViterbi;
	}
}

int CHLabViterbiRecog::ParseCommandLine(int argc, char** argv)
{
	m_compMVConfig.ParseCommandLine(argc, argv);

	return (0);
}

int CHLabViterbiRecog::Init(void)
{
	
    //Reading the phone list file
	m_phoneList.ReadPhoneFile(m_compMVConfig.GetPhoneFile());
//	m_phoneList.ListPhones();

	//Reading the dictionary file
	m_dict.SetPhoneList(&m_phoneList);
	m_dict.ReadDictFile(m_compMVConfig.GetDictFile());

	
	m_pHMM         = new CHLabHMM        [m_phoneList.GetNumPhones()];
	m_pViterbi     = new CHLabViterbi    [m_phoneList.GetNumPhones()];
	m_pWordViterbi = new CHLabWordViterbi[m_dict.GetNumDictWords()];


	/* Check whether we need to read the LM File or not */

	if (m_compMVConfig.GetLMFile() != NULL &&
		m_compMVConfig.GetLMWeight() > 0)
	{
		m_LM.ReadLM(m_compMVConfig.GetLMFile());
	}

	

	if (m_pHMM == NULL)
	{
		E_FATAL("Memoray allocation failure\n");
	}

	if (m_pViterbi == NULL)
	{
		E_FATAL("Memoray allocation failure\n");
	}

	if (m_pWordViterbi == NULL)
	{
		E_FATAL("Memory allocation failure\n");
	}

	int i;
	char acBuffer[255];

	for (i = 0; i < m_phoneList.GetNumPhones(); i++)
	{
		sprintf(acBuffer, "%d", i);
		m_pHMM[i].Init(39, m_compMVConfig.GetNumStatesPerHMM(), acBuffer);
		m_pHMM[i].ReadHMM(this->m_compMVConfig.GetModelDir());
	}

	std::map< std::string, vector<int> >           dicWords;
	std::map< std::string, vector<int> >::iterator itr;
	std::vector<int>                               vecPhoneSeq;
	std::vector<int>::iterator                     itr2;

	dicWords = m_dict.GetMapDicWords();

	i = 0;


	/*

		Building word-HMM for each word in dictionary

	*/
	for (itr = dicWords.begin(); itr != dicWords.end(); itr++, i++)
	{
		vecPhoneSeq = (*itr).second;

		m_pWordViterbi[i].Init(m_pHMM, 
			                  m_phoneList.GetNumPhones(), 
							  vecPhoneSeq, 
							  &m_testFeat); 
	}
	
	
	return (0);
}

// Processing the control file line by line
int CHLabViterbiRecog::Run(void)
{
	int i, j;

	int   iWordIndex;
	float fHighest;
	float fLH;
	int   iBackPointerFlag;
	float fOutLikelihood;
	float fInLikelihood;
	int   iOutBackPointer;
	int   iInBackPointer;
	int   iBackPointer;
	float fTh;
	float fTemp;
	float fTempHighest;



	float fOutWordLikelihood;
	int   iOutWordBackPointer;
	float fWordTemp;
	float fInWordLikelihood;
	int   iInWordBackPointer;
	float fWordTh;

	int   iPreviousWordIndex;
	int   iCurrentWordIndex;
	int   iNextWordIndex;
	float fLMLike;

	std::vector<int> vecResult;
	std::vector<int>::iterator itr;


	CHLabBPTableNode BPTableNode;

	m_testFeat.ReadFeatFile(m_compMVConfig.GetInputFile(), 13);

	/* time-synchronous */

	fTh     = -HLab_INFINITY;
	fWordTh = -HLab_INFINITY;

	for (i = 0; i < m_phoneList.GetNumPhones(); i++)
	{ 
		m_pViterbi[i].Init(&(m_pHMM[i]), &m_testFeat);
	}

	iInBackPointer     = 0;
	iInWordBackPointer = 0;
	iInWordBackPointer = 0;
	fInLikelihood      = -HLab_INFINITY;
	fInWordLikelihood  = -HLab_INFINITY;

	iPreviousWordIndex = -1;
	iCurrentWordIndex  = -1;

	std::string strPrevWord;
	std::string strCurrentWord;
	std::string strNextWord;
	char        acStrEntire[MAX_LINE_LEN];
	std::string strEntire;

	fLMLike = -HLab_INFINITY;

	LM_PROB lmProb;

	/* Time-synchronous processing */
	for (i = 0; i < m_testFeat.m_iLen; i++)
	{
		iWordIndex   = 0;
		fHighest     = -HLab_INFINITY;
		fTempHighest = -HLab_INFINITY;
		iBackPointer = 0;

		for (j = 0; j < m_dict.GetNumDictWords(); j++)
		{

			/* Let's calculate the trigram */

			if (iPreviousWordIndex >= 0)
			{
				strPrevWord    = m_dict.GetDictSymbol(iPreviousWordIndex);
			}
			else
			{
				strPrevWord.replace(0, strlen("<s>"), "<s>");
			}

			if (iCurrentWordIndex >= 0)
			{
				strCurrentWord = m_dict.GetDictSymbol(iCurrentWordIndex);
			}
			else
			{
				strCurrentWord.replace(0, strlen("<s>"), "<s>");
			}

			strNextWord    = m_dict.GetDictSymbol(j);

			if (iCurrentWordIndex >= 0)
			{
				sprintf(acStrEntire, "%s_%s_%s", strPrevWord.c_str(),
					                             strCurrentWord.c_str(), 
												 strNextWord.c_str());

				strEntire.clear();
				strEntire.replace(0, strlen(acStrEntire), acStrEntire);

				lmProb = m_LM.ComputeTrigramProb(strEntire);

				fLMLike = lmProb.m_fLogProb;

			
			}
			else
			{
				sprintf(acStrEntire, "%s_%s", strCurrentWord.c_str(), 
					                          strNextWord.c_str());


				strEntire.clear();
				strEntire.replace(0, strlen(acStrEntire), acStrEntire);

				lmProb = m_LM.ComputeBigramProb(strEntire);

				fLMLike = lmProb.m_fLogProb;

				
			}

			if (m_compMVConfig.GetLMWeight() != 0)
			{
				fLMLike = fLMLike * this->m_compMVConfig.GetLMWeight();
				
				fInWordLikelihood = fInWordLikelihood + fLMLike;
			}

			
			

			m_pWordViterbi[j].Run(&fOutWordLikelihood,
						          &iOutWordBackPointer,
								  &fWordTemp,
								  fInWordLikelihood,
								  iInWordBackPointer,
								  fWordTh);

			/* This is for finding the backpointer */
			/* Finding the highest likelihood at the non-emitting state */
			if (fHighest < fOutWordLikelihood)
			{
				/* Need to check whether it satisfies the FSN */
				iWordIndex = j;
				fHighest     = fOutWordLikelihood;
				iBackPointer = iOutWordBackPointer;
			}

			/* This is for finding the beam pruning */
			if (fWordTemp > fTempHighest)
			{
				fTempHighest = fWordTemp;
			}
			
		}

		if (fTempHighest <= -HLab_INFINITY)
		{
			E_WARN("Every thing is pruned. Increase the beam width\n");

			return (0);
		}

		fTh = fTempHighest - this->m_compMVConfig.GetBeam();
		fWordTh = fTh;

	
		
		if (fHighest > -HLab_INFINITY)
		{
			BPTableNode.m_iFrameIndex = i;
			BPTableNode.m_dPathScore  = fHighest;
			BPTableNode.m_iWordID     = iWordIndex;
			BPTableNode.m_iPredNodeID = iBackPointer;
			fInLikelihood             = fHighest; // - m_compMVConfig.GetWordInsertionPenalty();
			fInWordLikelihood         = fHighest - m_compMVConfig.GetWordInsertionPenalty();

			/* For the trigram processing */
			/* Let's traceback using the back pointer */
			/* Finding the previous word index and current word index */

			

			if (m_BPTable.m_vecBPNode.size() > 0)
			{
				iPreviousWordIndex = m_BPTable.m_vecBPNode[iBackPointer].m_iWordID;
			}
			else
			{
				iPreviousWordIndex = -1; // It is <s> case
			}
			iCurrentWordIndex  = iWordIndex;

			m_BPTable.push_back(BPTableNode);
			iInBackPointer = m_BPTable.GetNumBPNode();
			iInWordBackPointer = iInBackPointer;
		}
	}

	if (fHighest <=  -HLab_INFINITY)
	{
		E_WARN("Every thing is pruned. Increase the beam width\n");

		return (0);
	}

	if (iInBackPointer >= 1)
	{
	
		if (this->m_compMVConfig.GetPhoneNumberFlag() == 1)
		{
			m_BPTable.BackTrackingPhoneNumber(iInBackPointer - 1);
		}
		else
		{
			/*

				printing the result
			*/

			printf("[RESULT] ");

			vecResult = m_BPTable.BackTracking(iInBackPointer - 1);

			for (itr = vecResult.begin(); itr < vecResult.end(); itr++)
			{
				std::cout << m_dict.GetDictSymbol(*itr) << " ";
			}

			printf("\n");
		}
	}
	else
	{
		E_WARN("Every thing is pruned. Increase the beam width\n");

		return (0);
	}

	return (0);
}
